# Generated by Django 2.1.7 on 2019-03-01 06:37

from decimal import Decimal
from django.db import migrations, models
import django.db.models.deletion
import django.utils.timezone
import djeym.utils
import smart_selects.db_fields


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='CategoryPlacemark',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('title', models.CharField(help_text='Name of any geographic area - Country Name or City Name or District Name or The name of the resort coast, etc.', max_length=60, verbose_name='Title')),
                ('category_icon', models.CharField(blank=True, default='', help_text='https://fontawesome.com/icons?d=gallery&m=free - Example: &lt;i class="fab fa-android"&gt;&lt;/i&gt;', max_length=255, verbose_name='Category Icon (html tag)')),
                ('category_color', models.CharField(default='#00bfff', help_text='Applies to Checkboxes and Radio Buttons.', max_length=255, validators=[djeym.utils.validate_hex_color], verbose_name='Category color')),
                ('active', models.BooleanField(default=True, verbose_name='Active category')),
            ],
            options={
                'verbose_name': 'Category of placemarks',
                'verbose_name_plural': 'Category of placemarks',
                'ordering': ('title',),
            },
        ),
        migrations.CreateModel(
            name='CategoryPolygon',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('title', models.CharField(help_text='The name of the aggregate of any geographic area. Examples: Honduran cities or Hawaiian beaches or Residential complexes in Antarctica.', max_length=60, verbose_name='Title')),
                ('category_icon', models.CharField(blank=True, default='', help_text='https://fontawesome.com/icons?d=gallery&m=free - Example: &lt;i class="fab fa-android"&gt;&lt;/i&gt;', max_length=255, verbose_name='Category Icon (html tag)')),
                ('category_color', models.CharField(default='#00bfff', help_text='Applies to Checkboxes and Radio Buttons.', max_length=255, validators=[djeym.utils.validate_hex_color], verbose_name='Category color')),
                ('active', models.BooleanField(default=True, verbose_name='Active category')),
            ],
            options={
                'verbose_name': 'Category of territories',
                'verbose_name_plural': 'Category of territories',
                'ordering': ('title',),
            },
        ),
        migrations.CreateModel(
            name='CategoryPolyline',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('title', models.CharField(help_text='Examples: Tourist routes or Marathon routes etc.', max_length=60, verbose_name='Title')),
                ('category_icon', models.CharField(blank=True, default='', help_text='https://fontawesome.com/icons?d=gallery&m=free - Example: &lt;i class="fab fa-android"&gt;&lt;/i&gt;', max_length=255, verbose_name='Category Icon (html tag)')),
                ('category_color', models.CharField(default='#00bfff', help_text='Applies to Checkboxes and Radio Buttons.', max_length=255, validators=[djeym.utils.validate_hex_color], verbose_name='Category color')),
                ('active', models.BooleanField(default=True, verbose_name='Active category')),
            ],
            options={
                'verbose_name': 'Category of routes',
                'verbose_name_plural': 'Category of routes',
                'ordering': ('title',),
            },
        ),
        migrations.CreateModel(
            name='CounterID',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('num_id', models.PositiveIntegerField(default=1, verbose_name='ID')),
            ],
        ),
        migrations.CreateModel(
            name='CustomClusterIcon',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('svg', models.FileField(help_text='Only SVG files.', null=True, upload_to=djeym.utils.make_upload_path, validators=[djeym.utils.validate_svg], verbose_name='Icon')),
                ('title', models.CharField(default='', help_text='Example: Green Garden', max_length=60, unique=True, verbose_name='Cluster name')),
                ('size_width', models.PositiveSmallIntegerField(default=0, verbose_name='Icon width')),
                ('size_height', models.PositiveSmallIntegerField(default=0, verbose_name='Icon height')),
                ('offset_x', models.DecimalField(decimal_places=1, default=Decimal('0.0'), max_digits=3, verbose_name='Offset by axis - X')),
                ('offset_y', models.DecimalField(decimal_places=1, default=Decimal('0.0'), max_digits=3, verbose_name='Offset by axis - Y')),
            ],
            options={
                'verbose_name': 'Icons for clusters',
                'verbose_name_plural': 'Icons for clusters',
                'ordering': ('title', 'id'),
            },
        ),
        migrations.CreateModel(
            name='CustomMarkerIcon',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('title', models.CharField(default='', help_text='Example: Airport', max_length=60, verbose_name='Icon name')),
                ('svg', models.FileField(help_text='Only SVG files.', null=True, upload_to=djeym.utils.make_upload_path, validators=[djeym.utils.validate_svg], verbose_name='Icon')),
                ('size_width', models.PositiveSmallIntegerField(default=0, verbose_name='Icon width')),
                ('size_height', models.PositiveSmallIntegerField(default=0, verbose_name='Icon height')),
                ('offset_x', models.DecimalField(decimal_places=1, default=Decimal('0.0'), help_text='Left, right - First time is automatically calculated.', max_digits=3, verbose_name='Offset by axis - X')),
                ('offset_y', models.DecimalField(decimal_places=1, default=Decimal('0.0'), help_text='Up, down - First time is automatically calculated.', max_digits=3, verbose_name='Offset by axis - Y')),
                ('active', models.BooleanField(default=True, help_text='If the project uses 2-3 icons, it makes sense to disable the rest to optimize the download.', verbose_name='Active icon')),
                ('slug', models.SlugField(blank=True, max_length=255, null=True)),
            ],
            options={
                'verbose_name': 'Icons for markers',
                'verbose_name_plural': 'Icons for markers',
                'ordering': ('title', 'id'),
            },
        ),
        migrations.CreateModel(
            name='ExternalModules',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('roundtheme', models.BooleanField(default=False, verbose_name='Round map controls theme')),
                ('heatmap', models.BooleanField(default=False, verbose_name='Heatmap')),
                ('areacalculation', models.BooleanField(default=True, verbose_name='Area Calculation')),
            ],
            options={
                'verbose_name': 'External modules',
                'verbose_name_plural': 'External modules',
            },
        ),
        migrations.CreateModel(
            name='GeneralSettings',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('clustering_edit', models.BooleanField(default=True, verbose_name='Сlustering in editor')),
                ('clustering_site', models.BooleanField(default=True, verbose_name='Сlustering on site')),
                ('cluster_layout', models.CharField(choices=[('cluster#balloonTwoColumns', 'Two Columns'), ('cluster#balloonCarousel', 'Carousel')], default='cluster#balloonTwoColumns', max_length=255, verbose_name='Layout for the cluster information window')),
                ('cluster_icon_content', models.BooleanField(default=True, verbose_name='Display the number of objects in the cluster icon')),
                ('cluster_icon_content_bg_color', models.CharField(default='#ffffff', max_length=255, validators=[djeym.utils.validate_hex_color], verbose_name='Background color')),
                ('cluster_icon_content_txt_color', models.CharField(default='#333333', max_length=255, validators=[djeym.utils.validate_hex_color], verbose_name='Text color')),
                ('disable_site_panel', models.BooleanField(default=False, verbose_name='Disable the panel on the site (button with the image of the gear and the eye in the center)')),
            ],
            options={
                'verbose_name': 'General settings',
                'verbose_name_plural': 'General settings',
            },
        ),
        migrations.CreateModel(
            name='HeatmapSettings',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('radius', models.PositiveIntegerField(default=10, verbose_name='Point radius of influence (px)')),
                ('dissipating', models.BooleanField(default=False, help_text="Enable - Disperse points on higher zoom levels according to radius. Disable - Don't disperse.", verbose_name='Disperse points')),
                ('opacity', models.CharField(choices=[('0', '0'), ('0.1', '0.1'), ('0.2', '0.2'), ('0.3', '0.3'), ('0.4', '0.4'), ('0.5', '0.5'), ('0.6', '0.6'), ('0.7', '0.7'), ('0.8', '0.8'), ('0.9', '0.9'), ('1', '1')], default='0.8', max_length=255, validators=[djeym.utils.validate_transparency], verbose_name='Transparency of heat layer')),
                ('intensity', models.CharField(choices=[('0', '0'), ('0.1', '0.1'), ('0.2', '0.2'), ('0.3', '0.3'), ('0.4', '0.4'), ('0.5', '0.5'), ('0.6', '0.6'), ('0.7', '0.7'), ('0.8', '0.8'), ('0.9', '0.9'), ('1', '1')], default='0.2', max_length=255, validators=[djeym.utils.validate_transparency], verbose_name='Intensity of median point')),
                ('gradient_color1', models.CharField(default='#56db40b3', max_length=255, verbose_name='gradient color 1')),
                ('gradient_color2', models.CharField(default='#ffd21ecc', max_length=255, verbose_name='gradient color 2')),
                ('gradient_color3', models.CharField(default='#ed4543e6', max_length=255, verbose_name='gradient color 3')),
                ('gradient_color4', models.CharField(default='#b22222', max_length=255, verbose_name='gradient color 4')),
            ],
            options={
                'verbose_name': 'Heatmap',
                'verbose_name_plural': 'Heatmap',
            },
        ),
        migrations.CreateModel(
            name='HeatPoint',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('title', models.CharField(blank=True, default='', max_length=60, verbose_name='Place name')),
                ('weight', models.PositiveIntegerField(blank=True, default=0, help_text='Examples: Average property value on a district site or Number of cameras installed on a building, etc.', verbose_name='Weight')),
                ('coordinates', models.CharField(default='[0,0]', max_length=255, verbose_name='Coordinates')),
                ('active', models.BooleanField(default=True, verbose_name='Active heat point')),
                ('slug', models.SlugField(blank=True, max_length=255, null=True, unique=True)),
                ('json_code', models.TextField(blank=True, default='{"type": "Feature", "id": 0, "geometry": {"type": "Point", "coordinates": []}, "properties": {"weight": 0}}', verbose_name='JSON')),
            ],
            options={
                'verbose_name': 'Thermal points',
                'verbose_name_plural': 'Thermal points',
            },
        ),
        migrations.CreateModel(
            name='IconCollection',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('title', models.CharField(default='', help_text='Example: Light golden Avocado', max_length=60, unique=True, verbose_name='Collection name')),
                ('slug', models.SlugField(blank=True, max_length=255, null=True, unique=True)),
            ],
            options={
                'verbose_name': 'Icon collections for markers',
                'verbose_name_plural': 'Icon collections for markers',
                'ordering': ('title', 'id'),
            },
        ),
        migrations.CreateModel(
            name='Map',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('title', models.CharField(help_text='Examples: Supermarkets of the city of Kharkov or Hawaii - Oahu Island or Luxury hotels in Honolulu or Parking of primitive people, etc.', max_length=60, unique=True, verbose_name='Title')),
                ('active', models.BooleanField(default=True, verbose_name='Active map')),
                ('zoom', models.PositiveSmallIntegerField(choices=[(0, 0), (1, 1), (2, 2), (3, 3), (4, 4), (5, 5), (6, 6), (7, 7), (8, 8), (9, 9), (10, 10), (11, 11), (12, 12), (13, 13), (14, 14), (15, 15), (16, 16), (17, 17), (18, 18), (19, 19), (20, 20), (21, 21), (22, 22), (23, 23)], default=3, verbose_name='Zoom')),
                ('latitude', models.CharField(default='0', help_text='Latitude center of the map.', max_length=255, validators=[djeym.utils.validate_coordinates], verbose_name='Latitude')),
                ('longitude', models.CharField(default='0', help_text='Longitude center of the map.', max_length=255, validators=[djeym.utils.validate_coordinates], verbose_name='Longitude')),
                ('slug', models.SlugField(blank=True, max_length=255, null=True, unique=True)),
                ('created_at', models.DateTimeField(auto_now_add=True, verbose_name='Created at')),
                ('updated_at', models.DateTimeField(auto_now=True, verbose_name='Updated at')),
                ('icon_cluster', models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='placemark_custom_cluster', to='djeym.CustomClusterIcon', verbose_name='Icon for cluster')),
                ('icon_collection', models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='map_icon_collection', to='djeym.IconCollection', verbose_name='Icon collection for markers')),
            ],
            options={
                'verbose_name': 'Map',
                'verbose_name_plural': 'Map',
                'ordering': ('title',),
            },
        ),
        migrations.CreateModel(
            name='MapControls',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('geolocation', models.BooleanField(default=True, verbose_name='Geolocation')),
                ('search', models.BooleanField(default=True, verbose_name='Search by map')),
                ('provider', models.BooleanField(default=True, verbose_name='Search by organization name')),
                ('route', models.BooleanField(default=True, verbose_name='Routing panel')),
                ('traffic', models.BooleanField(default=True, verbose_name='Traffic jams')),
                ('typeselector', models.BooleanField(default=True, verbose_name='Map Layer Switch')),
                ('fullscreen', models.BooleanField(default=True, verbose_name='Full screen mode')),
                ('zoom', models.BooleanField(default=True, verbose_name='Zoom')),
                ('ruler', models.BooleanField(default=True, verbose_name='Ruler')),
                ('maptype', models.CharField(choices=[('yandex#map', 'Scheme'), ('yandex#satellite', 'Satellite'), ('yandex#hybrid', 'Hybrid')], default='yandex#map', max_length=255, verbose_name='Map type')),
                ('ymap', models.OneToOneField(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='controls', to='djeym.Map', verbose_name='Map')),
            ],
            options={
                'verbose_name_plural': 'Map Controls',
            },
        ),
        migrations.CreateModel(
            name='Placemark',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('header', models.TextField(default='', verbose_name='Place name')),
                ('body', models.TextField(blank=True, default='', verbose_name='Content')),
                ('footer', models.TextField(blank=True, default='', verbose_name='Footer')),
                ('icon_name', models.CharField(default='', max_length=255, verbose_name='Icon name')),
                ('coordinates', models.CharField(default='[0,0]', max_length=255, verbose_name='Coordinates')),
                ('like', models.PositiveIntegerField(blank=True, default=0, verbose_name='Like')),
                ('dislike', models.PositiveIntegerField(blank=True, default=0, verbose_name='Dislike')),
                ('active', models.BooleanField(default=True, verbose_name='Active placemark')),
                ('json_code', models.TextField(blank=True, default='{"type": "Feature", "id": 0, "geometry": {"type": "Point", "coordinates": []}, "properties": {"balloonContentHeader": "", "balloonContentBody": "", "balloonContentFooter": "", "id": 0, "categoryID": 0, "subCategoryIDs": [], "iconName": ""}, "options": {"iconLayout": "default#image", "iconImageHref": "", "iconImageSize": [], "iconImageOffset": []}}', verbose_name='JSON')),
                ('created_at', models.DateTimeField(auto_now_add=True, verbose_name='Created at')),
                ('updated_at', models.DateTimeField(auto_now=True, verbose_name='Updated at')),
                ('category', smart_selects.db_fields.ChainedForeignKey(auto_choose=True, chained_field='ymap', chained_model_field='ymap', on_delete=django.db.models.deletion.CASCADE, related_name='placemark_category', to='djeym.CategoryPlacemark', verbose_name='Category')),
            ],
            options={
                'verbose_name': 'Placemark',
                'verbose_name_plural': 'Placemarks',
                'ordering': ('-id',),
            },
        ),
        migrations.CreateModel(
            name='Polygon',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('header', models.TextField(default='', verbose_name='Territory name')),
                ('body', models.TextField(blank=True, default='', verbose_name='Content')),
                ('footer', models.TextField(blank=True, default='', verbose_name='Footer')),
                ('stroke_width', models.PositiveIntegerField(default=2, verbose_name='Stroke width')),
                ('stroke_color', models.CharField(default='#1e98ff', max_length=255, validators=[djeym.utils.validate_hex_color], verbose_name='Line color')),
                ('stroke_opacity', models.CharField(choices=[('0', '0'), ('0.1', '0.1'), ('0.2', '0.2'), ('0.3', '0.3'), ('0.4', '0.4'), ('0.5', '0.5'), ('0.6', '0.6'), ('0.7', '0.7'), ('0.8', '0.8'), ('0.9', '0.9'), ('1', '1')], default='0.9', max_length=255, validators=[djeym.utils.validate_transparency], verbose_name='Opacity line')),
                ('fill_color', models.CharField(default='#1e98ff', max_length=255, validators=[djeym.utils.validate_hex_color], verbose_name='Fill color')),
                ('fill_opacity', models.CharField(choices=[('0', '0'), ('0.1', '0.1'), ('0.2', '0.2'), ('0.3', '0.3'), ('0.4', '0.4'), ('0.5', '0.5'), ('0.6', '0.6'), ('0.7', '0.7'), ('0.8', '0.8'), ('0.9', '0.9'), ('1', '1')], default='0.9', max_length=255, validators=[djeym.utils.validate_transparency], verbose_name='Fill opacity')),
                ('coordinates', models.TextField(default='', verbose_name='Coordinates')),
                ('like', models.PositiveIntegerField(blank=True, default=0, verbose_name='Like')),
                ('dislike', models.PositiveIntegerField(blank=True, default=0, verbose_name='Dislike')),
                ('active', models.BooleanField(default=True, verbose_name='Active territory')),
                ('json_code', models.TextField(blank=True, default='{"type": "Feature", "id": 0, "geometry": {"type": "Polygon", "coordinates": []}, "properties": {"balloonContentHeader": "", "balloonContentBody": "", "balloonContentFooter": "", "id": 0, "categoryID": 0}, "options": {"strokeWidth": 2, "strokeColor": "#1e98ff", "strokeOpacity": 0.9, "fillColor": "#1e98ff", "fillOpacity": 0.9}}', verbose_name='JSON')),
                ('created_at', models.DateTimeField(auto_now_add=True, verbose_name='Created at')),
                ('updated_at', models.DateTimeField(auto_now=True, verbose_name='Updated at')),
                ('category', smart_selects.db_fields.ChainedForeignKey(auto_choose=True, chained_field='ymap', chained_model_field='ymap', on_delete=django.db.models.deletion.CASCADE, related_name='polygon_category', to='djeym.CategoryPolygon', verbose_name='Category')),
                ('ymap', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='polygon_map', to='djeym.Map', verbose_name='Map')),
            ],
            options={
                'verbose_name': 'Territorys',
                'verbose_name_plural': 'Territorys',
                'ordering': ('-id',),
            },
        ),
        migrations.CreateModel(
            name='Polyline',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('header', models.TextField(default='', verbose_name='Route name')),
                ('body', models.TextField(blank=True, default='', verbose_name='Content')),
                ('footer', models.TextField(blank=True, default='', verbose_name='Footer')),
                ('stroke_width', models.PositiveIntegerField(default=5, verbose_name='Stroke width')),
                ('stroke_color', models.CharField(default='#1e98ff', max_length=255, validators=[djeym.utils.validate_hex_color], verbose_name='Line color')),
                ('stroke_opacity', models.CharField(choices=[('0', '0'), ('0.1', '0.1'), ('0.2', '0.2'), ('0.3', '0.3'), ('0.4', '0.4'), ('0.5', '0.5'), ('0.6', '0.6'), ('0.7', '0.7'), ('0.8', '0.8'), ('0.9', '0.9'), ('1', '1')], default='0.9', max_length=255, validators=[djeym.utils.validate_transparency], verbose_name='Opacity line')),
                ('coordinates', models.TextField(default='', verbose_name='Coordinates')),
                ('like', models.PositiveIntegerField(blank=True, default=0, verbose_name='Like')),
                ('dislike', models.PositiveIntegerField(blank=True, default=0, verbose_name='Dislike')),
                ('active', models.BooleanField(default=True, verbose_name='Active route')),
                ('json_code', models.TextField(blank=True, default='{"type": "Feature", "id": 0, "geometry": {"type": "LineString", "coordinates": []}, "properties": {"balloonContentHeader": "", "balloonContentBody": "", "balloonContentFooter": "", "id": 0, "categoryID": 0}, "options": {"strokeWidth": 5, "strokeColor": "#1e98ff", "strokeOpacity": 0.9}}', verbose_name='JSON')),
                ('created_at', models.DateTimeField(auto_now_add=True, verbose_name='Created at')),
                ('updated_at', models.DateTimeField(auto_now=True, verbose_name='Updated at')),
                ('category', smart_selects.db_fields.ChainedForeignKey(auto_choose=True, chained_field='ymap', chained_model_field='ymap', on_delete=django.db.models.deletion.CASCADE, related_name='polyline_category', to='djeym.CategoryPolyline', verbose_name='Category')),
                ('ymap', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='polyline_map', to='djeym.Map', verbose_name='Map')),
            ],
            options={
                'verbose_name': 'Route',
                'verbose_name_plural': 'Routes',
                'ordering': ('-id',),
            },
        ),
        migrations.CreateModel(
            name='Preset',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('title', models.CharField(default='', max_length=60, verbose_name='Title')),
                ('icon', models.CharField(blank=True, default='<i class="far fa-smile"></i>', help_text='https://fontawesome.com/icons?d=gallery&m=free - Example: &lt;i class="fab fa-android"&gt;&lt;/i&gt;', max_length=255, verbose_name='Icon (html tag)')),
                ('html', models.TextField(default='<p style="color:#e91e63;">iPreset</p>', verbose_name='HTML')),
                ('js', models.TextField(blank=True, default='', help_text='JavaScript and jQuery', verbose_name='JS')),
                ('description', models.TextField(default='<div>Описание</div>\n<div style="color:#607D8B;">Description</div>', help_text='Brief description of the preset.', verbose_name='Description')),
                ('autoheader', models.BooleanField(default=False, verbose_name='Automatically add to end of header')),
                ('autobody', models.BooleanField(default=False, verbose_name='Automatically add to end of description')),
                ('autofooter', models.BooleanField(default=False, verbose_name='Automatically add to end of footer')),
                ('placemark', models.BooleanField(default=True, verbose_name='Markers')),
                ('polyline', models.BooleanField(default=True, verbose_name='Routes')),
                ('polygon', models.BooleanField(default=True, verbose_name='Territorys')),
                ('position', models.PositiveSmallIntegerField(blank=True, default=0, verbose_name='Position')),
                ('slug', models.SlugField(blank=True, max_length=255, null=True)),
                ('ymap', models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='presets', to='djeym.Map', verbose_name='Map')),
            ],
            options={
                'verbose_name': 'Preset',
                'verbose_name_plural': 'Presets',
                'ordering': ('position',),
            },
        ),
        migrations.CreateModel(
            name='Statistics',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('obj_type', models.CharField(default='', max_length=255, verbose_name='Object type')),
                ('obj_id', models.PositiveIntegerField(default=0, verbose_name='Object ID')),
                ('ip', models.GenericIPAddressField(null=True, verbose_name='IP-address')),
                ('likes', models.BooleanField(default=False, verbose_name='Likes')),
                ('timestamp', models.DateTimeField(default=django.utils.timezone.now, verbose_name='Date and Time')),
            ],
            options={
                'verbose_name': 'Record',
                'verbose_name_plural': 'Statistics',
            },
        ),
        migrations.CreateModel(
            name='SubCategoryPlacemark',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('title', models.CharField(help_text='Subcategory - This is what complements the categories of markers. Examples: Parking or Bicycle Parking or ATM or Cafe or Wi-Fi or Playground, etc.', max_length=60, verbose_name='Title')),
                ('category_icon', models.CharField(blank=True, default='', help_text='https://fontawesome.com/icons?d=gallery&m=free - Example: &lt;i class="fab fa-android"&gt;&lt;/i&gt;', max_length=255, verbose_name='Category Icon (html tag)')),
                ('category_color', models.CharField(default='#ffcc00', help_text='Applies to Checkboxes and Radio Buttons.', max_length=255, validators=[djeym.utils.validate_hex_color], verbose_name='Category color')),
                ('active', models.BooleanField(default=True, verbose_name='Active category')),
                ('ymap', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='subcategory_placemark', to='djeym.Map', verbose_name='Map')),
            ],
            options={
                'verbose_name': 'Subcategory of placemarks',
                'verbose_name_plural': 'Subcategory of placemarks',
                'ordering': ('title',),
            },
        ),
        migrations.CreateModel(
            name='TileSource',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('title', models.CharField(default='', max_length=255, unique=True, verbose_name='Title')),
                ('maxzoom', models.PositiveSmallIntegerField(choices=[(0, 0), (1, 1), (2, 2), (3, 3), (4, 4), (5, 5), (6, 6), (7, 7), (8, 8), (9, 9), (10, 10), (11, 11), (12, 12), (13, 13), (14, 14), (15, 15), (16, 16), (17, 17), (18, 18), (19, 19), (20, 20), (21, 21), (22, 22), (23, 23)], default=12, verbose_name='Maximum zoom')),
                ('minzoom', models.PositiveSmallIntegerField(choices=[(0, 0), (1, 1), (2, 2), (3, 3), (4, 4), (5, 5), (6, 6), (7, 7), (8, 8), (9, 9), (10, 10), (11, 11), (12, 12), (13, 13), (14, 14), (15, 15), (16, 16), (17, 17), (18, 18), (19, 19), (20, 20), (21, 21), (22, 22), (23, 23)], default=0, verbose_name='Minimum zoom')),
                ('source', models.TextField(default='', verbose_name='Source')),
                ('screenshot', models.ImageField(help_text='Recommended size - Width=360 x Height=180', null=True, upload_to=djeym.utils.make_upload_path, validators=[djeym.utils.validate_image], verbose_name='Screenshot')),
                ('copyrights', models.TextField(blank=True, default='', verbose_name='Copyrights')),
                ('site', models.URLField(blank=True, default='', verbose_name='Site')),
                ('apikey', models.CharField(blank=True, default='', help_text='API key or access_token', max_length=255, verbose_name='API Key')),
                ('note', models.TextField(blank=True, default='', verbose_name='Note')),
                ('slug', models.SlugField(blank=True, max_length=255, null=True, unique=True)),
            ],
            options={
                'verbose_name': 'Tile Source',
                'verbose_name_plural': 'Tile Sources',
                'ordering': ('title',),
            },
        ),
        migrations.AddField(
            model_name='placemark',
            name='subcategories',
            field=smart_selects.db_fields.ChainedManyToManyField(blank=True, chained_field='ymap', chained_model_field='ymap', related_name='placemark_subcategories', to='djeym.SubCategoryPlacemark'),
        ),
        migrations.AddField(
            model_name='placemark',
            name='ymap',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='placemark_map', to='djeym.Map', verbose_name='Map'),
        ),
        migrations.AddField(
            model_name='map',
            name='tile',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='maptile', to='djeym.TileSource', verbose_name='Tile Source'),
        ),
        migrations.AddField(
            model_name='heatpoint',
            name='ymap',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='heat_point', to='djeym.Map', verbose_name='Map'),
        ),
        migrations.AddField(
            model_name='heatmapsettings',
            name='ymap',
            field=models.OneToOneField(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='heatmap_settings', to='djeym.Map', verbose_name='Map'),
        ),
        migrations.AddField(
            model_name='generalsettings',
            name='ymap',
            field=models.OneToOneField(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='general_settings', to='djeym.Map', verbose_name='Map'),
        ),
        migrations.AddField(
            model_name='externalmodules',
            name='ymap',
            field=models.OneToOneField(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='external_modules', to='djeym.Map', verbose_name='Map'),
        ),
        migrations.AddField(
            model_name='custommarkericon',
            name='icon_collection',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='icons', to='djeym.IconCollection', verbose_name='Icon Collection'),
        ),
        migrations.AddField(
            model_name='categorypolyline',
            name='ymap',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='category_polyline', to='djeym.Map', verbose_name='Map'),
        ),
        migrations.AddField(
            model_name='categorypolygon',
            name='ymap',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='category_polygon', to='djeym.Map', verbose_name='Map'),
        ),
        migrations.AddField(
            model_name='categoryplacemark',
            name='ymap',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='category_placemark', to='djeym.Map', verbose_name='Map'),
        ),
    ]



# Generated by Django 2.2.5 on 2019-09-20 13:45

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('djeym', '0004_auto_20190330_1458'),
    ]

    operations = [
        migrations.AlterField(
            model_name='heatpoint',
            name='json_code',
            field=models.TextField(blank=True, default='{"type": "Feature", "id": 0, "geometry": {"type": "Point", "coordinates": []}, "properties": {"weight": 0}}', editable=False, verbose_name='JSON'),
        ),
        migrations.AlterField(
            model_name='placemark',
            name='json_code',
            field=models.TextField(blank=True, default='{"type": "Feature", "id": 0, "geometry": {"type": "Point", "coordinates": []}, "properties": {"balloonContentHeader": "", "balloonContentBody": "", "balloonContentFooter": "", "id": 0, "categoryID": 0, "subCategoryIDs": [], "iconName": ""}, "options": {"iconLayout": "default#image", "iconImageHref": "", "iconImageSize": [], "iconImageOffset": []}}', editable=False, verbose_name='JSON'),
        ),
        migrations.AlterField(
            model_name='polygon',
            name='json_code',
            field=models.TextField(blank=True, default='{"type": "Feature", "id": 0, "geometry": {"type": "Polygon", "coordinates": []}, "properties": {"balloonContentHeader": "", "balloonContentBody": "", "balloonContentFooter": "", "id": 0, "categoryID": 0}, "options": {"strokeWidth": 2, "strokeColor": "#1e98ff", "strokeOpacity": 0.9, "fillColor": "#1e98ff", "fillOpacity": 0.9}}', editable=False, verbose_name='JSON'),
        ),
        migrations.AlterField(
            model_name='polyline',
            name='json_code',
            field=models.TextField(blank=True, default='{"type": "Feature", "id": 0, "geometry": {"type": "LineString", "coordinates": []}, "properties": {"balloonContentHeader": "", "balloonContentBody": "", "balloonContentFooter": "", "id": 0, "categoryID": 0}, "options": {"strokeWidth": 5, "strokeColor": "#1e98ff", "strokeOpacity": 0.9}}', editable=False, verbose_name='JSON'),
        ),
    ]



# -*- coding: utf-8 -*-
from ckeditor_uploader.widgets import CKEditorUploadingWidget
from django.contrib import admin
from django.db import models

from .forms import CenterMapForm, OffsetCustomIconForm
from .models import (CategoryPlacemark, CategoryPolygon, CategoryPolyline,
                     CustomClusterIcon, CustomMarkerIcon, ExternalModules,
                     GeneralSettings, HeatmapSettings, HeatPoint,
                     IconCollection, LoadIndicator, Map, MapControls,
                     Placemark, Polygon, Polyline, Preset, Statistics,
                     SubCategoryPlacemark, TileSource)
from .utils import get_icon_font_plugin
from .widgets import AdminFileThumbWidget, ColorPickerWidget

DJEYM_YMAPS_ICONS_FOR_CATEGORIES = get_icon_font_plugin()


@admin.register(TileSource)
class TileSourceAdmin(admin.ModelAdmin):
    # ckeditor_change_form.html - Used by default.
    change_form_template = 'djeym/admin/ckeditor_change_form.html'
    change_list_template = 'djeym/admin/tile_source_change_list.html'
    list_display = ('title', 'admin_thumbnail', 'maxzoom', 'minzoom', 'slug')
    list_display_links = ('title', 'admin_thumbnail')
    readonly_fields = ('slug',)

    formfield_overrides = {
        models.ImageField: {'widget': AdminFileThumbWidget()},
    }

    class Media:
        css = {
            'all': [
                '/static/djeym/css/djeym_admin.min.css',
            ]
        }

        js = [
            '/static/djeym/js/jquery-3.3.1.min.js',
            '/static/djeym/js/import_export.min.js'
        ]


class MapControlsInline(admin.TabularInline):
    model = MapControls
    can_delete = False
    classes = ['collapse']


class ExternalModulesInline(admin.StackedInline):
    model = ExternalModules
    classes = ['collapse']
    can_delete = False


class GeneralSettingsInline(admin.StackedInline):
    model = GeneralSettings
    can_delete = False
    classes = ['collapse']
    exclude = ('cluster_icon_content_bg_color',
               'cluster_icon_content_txt_color')


"""
class HeatmapSettingsInline(admin.StackedInline):
    model = HeatmapSettings
    can_delete = False
"""


class PresetInline(admin.StackedInline):
    model = Preset
    extra = 0
    exclude = ('slug',)
    classes = ['collapse']


@admin.register(Statistics)
class StatisticsAdmin(admin.ModelAdmin):
    # ckeditor_change_form.html - Used by default.
    change_form_template = 'djeym/admin/ckeditor_change_form.html'
    list_display = ('obj_type', 'obj_id', 'ip', 'timestamp')
    readonly_fields = ('likes',)

    class Media:
        css = {
            'all': ['/static/djeym/css/djeym_admin.min.css']
        }


@admin.register(Map)
class MapAdmin(admin.ModelAdmin):
    form = CenterMapForm
    change_form_template = 'djeym/admin/center_map_change_form.html'
    list_display = ('title', 'get_status_heatmap', 'get_custom_cluster',
                    'get_custom_marker_icon', 'get_tile_screenshot', 'get_load_indicator',
                    'zoom', 'slug', 'active')
    list_display_links = ('title', 'get_custom_cluster', 'get_status_heatmap',
                          'get_custom_marker_icon', 'get_tile_screenshot',
                          'get_load_indicator')
    list_editable = ('active',)
    readonly_fields = ('slug',)
    inlines = (MapControlsInline, ExternalModulesInline,
               GeneralSettingsInline, PresetInline)

    class Media:
        css = {
            'all': [
                '/static/djeym/css/djeym_admin.min.css',
            ]
        }

        js = [
            '/static/djeym/js/jquery-3.3.1.min.js',
            '/static/djeym/js/admin_view_icons.min.js',
        ]
        css['all'].extend(DJEYM_YMAPS_ICONS_FOR_CATEGORIES[0])
        js.extend(DJEYM_YMAPS_ICONS_FOR_CATEGORIES[1])


@admin.register(CategoryPlacemark)
class CategoryPlacemarkAdmin(admin.ModelAdmin):
    # ckeditor_change_form.html - Used by default.
    change_form_template = 'djeym/admin/ckeditor_change_form.html'
    list_display = ('get_title', 'get_category_icon', 'get_category_color',
                    'get_map_name', 'active')
    list_display_links = ('get_title', 'get_category_icon')
    list_editable = ('active',)
    list_filter = ('ymap',)

    def formfield_for_dbfield(self, db_field, **kwargs):
        target_field_name = db_field.name
        if target_field_name == 'category_color' or target_field_name == 'cluster_color':
            kwargs['widget'] = ColorPickerWidget()
        return super(CategoryPlacemarkAdmin, self).formfield_for_dbfield(db_field, **kwargs)

    formfield_overrides = {
        models.FileField: {'widget': AdminFileThumbWidget()},
    }

    class Media:
        css = {
            'all': ['/static/djeym/css/djeym_admin.min.css']
        }

        js = [
            '/static/djeym/js/jquery-3.3.1.min.js',
            '/static/djeym/js/admin_view_icons.min.js',
        ]
        css['all'].extend(DJEYM_YMAPS_ICONS_FOR_CATEGORIES[0])
        js.extend(DJEYM_YMAPS_ICONS_FOR_CATEGORIES[1])


@admin.register(SubCategoryPlacemark)
class SubCategoryPlacemarkAdmin(admin.ModelAdmin):
    # ckeditor_change_form.html - Used by default.
    change_form_template = 'djeym/admin/ckeditor_change_form.html'
    list_display = ('get_title', 'get_category_icon', 'get_category_color',
                    'get_map_name', 'active')
    list_display_links = ('get_title', 'get_category_icon')
    list_editable = ('active',)
    list_filter = ('ymap',)

    def formfield_for_dbfield(self, db_field, **kwargs):
        if db_field.name == 'category_color':
            kwargs['widget'] = ColorPickerWidget()
        return super(SubCategoryPlacemarkAdmin, self).formfield_for_dbfield(db_field, **kwargs)

    class Media:
        css = {
            'all': ['/static/djeym/css/djeym_admin.min.css']
        }

        js = [
            '/static/djeym/js/jquery-3.3.1.min.js',
            '/static/djeym/js/admin_view_icons.min.js',
        ]
        css['all'].extend(DJEYM_YMAPS_ICONS_FOR_CATEGORIES[0])
        js.extend(DJEYM_YMAPS_ICONS_FOR_CATEGORIES[1])


@admin.register(CategoryPolyline)
class CategoryPolylineAdmin(admin.ModelAdmin):
    # ckeditor_change_form.html - Used by default.
    change_form_template = 'djeym/admin/ckeditor_change_form.html'
    list_display = ('get_title', 'get_category_icon', 'get_category_color',
                    'get_map_name', 'active')
    list_display_links = ('get_title', 'get_category_icon')
    list_editable = ('active',)
    list_filter = ('ymap',)

    def formfield_for_dbfield(self, db_field, **kwargs):
        if db_field.name == 'category_color':
            kwargs['widget'] = ColorPickerWidget()
        return super(CategoryPolylineAdmin, self).formfield_for_dbfield(db_field, **kwargs)

    class Media:
        css = {
            'all': ['/static/djeym/css/djeym_admin.min.css']
        }

        js = [
            '/static/djeym/js/jquery-3.3.1.min.js',
            '/static/djeym/js/admin_view_icons.min.js',
        ]
        css['all'].extend(DJEYM_YMAPS_ICONS_FOR_CATEGORIES[0])
        js.extend(DJEYM_YMAPS_ICONS_FOR_CATEGORIES[1])


@admin.register(CategoryPolygon)
class CategoryPolygonAdmin(admin.ModelAdmin):
    # ckeditor_change_form.html - Used by default.
    change_form_template = 'djeym/admin/ckeditor_change_form.html'
    list_display = ('get_title', 'get_category_icon', 'get_category_color',
                    'get_map_name', 'active')
    list_display_links = ('get_title', 'get_category_icon')
    list_editable = ('active',)
    list_filter = ('ymap',)

    def formfield_for_dbfield(self, db_field, **kwargs):
        if db_field.name == 'category_color':
            kwargs['widget'] = ColorPickerWidget()
        return super(CategoryPolygonAdmin, self).formfield_for_dbfield(db_field, **kwargs)

    class Media:
        css = {
            'all': ['/static/djeym/css/djeym_admin.min.css']
        }

        js = [
            '/static/djeym/js/jquery-3.3.1.min.js',
            '/static/djeym/js/admin_view_icons.min.js',
        ]
        css['all'].extend(DJEYM_YMAPS_ICONS_FOR_CATEGORIES[0])
        js.extend(DJEYM_YMAPS_ICONS_FOR_CATEGORIES[1])


@admin.register(Placemark)
class PlacemarkAdmin(admin.ModelAdmin):
    change_form_template = 'djeym/admin/ckeditor_change_form.html'
    list_display = ('__str__', 'ymap', 'category', 'active')
    list_filter = ('ymap',)
    readonly_fields = ('coordinates', 'icon_name')
    list_editable = ('active',)

    formfield_overrides = {
        models.TextField: {'widget': CKEditorUploadingWidget(config_name='djeym')},
    }

    def formfield_for_dbfield(self, db_field, **kwargs):
        if db_field.name == 'marker_color':
            kwargs['widget'] = ColorPickerWidget()
        return super(PlacemarkAdmin, self).formfield_for_dbfield(db_field, **kwargs)

    class Media:
        css = {
            'all': [
                '/static/djeym/css/djeym_admin.min.css',
            ]
        }

        js = (
            '/static/djeym/js/jquery-3.3.1.min.js',
            '/static/djeym/js/ckeditor_resize_image.min.js',
        )


@admin.register(Polyline)
class PolylineAdmin(admin.ModelAdmin):
    change_form_template = 'djeym/admin/ckeditor_change_form.html'
    list_display = ('__str__', 'active')
    list_filter = ('ymap',)
    readonly_fields = ('coordinates',)
    list_editable = ('active',)

    formfield_overrides = {
        models.TextField: {'widget': CKEditorUploadingWidget(config_name='djeym')},
    }

    def formfield_for_dbfield(self, db_field, **kwargs):
        if db_field.name == 'stroke_color':
            kwargs['widget'] = ColorPickerWidget()
        return super(PolylineAdmin, self).formfield_for_dbfield(db_field, **kwargs)

    class Media:
        css = {
            'all': [
                '/static/djeym/css/djeym_admin.min.css',
            ]
        }

        js = (
            '/static/djeym/js/jquery-3.3.1.min.js',
            '/static/djeym/js/ckeditor_resize_image.min.js',
        )


@admin.register(Polygon)
class PolygonAdmin(admin.ModelAdmin):
    change_form_template = 'djeym/admin/ckeditor_change_form.html'
    list_display = ('__str__', 'active')
    list_filter = ('ymap',)
    readonly_fields = ('coordinates',)
    list_editable = ('active',)

    formfield_overrides = {
        models.TextField: {'widget': CKEditorUploadingWidget(config_name='djeym')},
    }

    def formfield_for_dbfield(self, db_field, **kwargs):
        if db_field.name == 'stroke_color' or db_field.name == 'fill_color':
            kwargs['widget'] = ColorPickerWidget()
        return super(PolygonAdmin, self).formfield_for_dbfield(db_field, **kwargs)

    class Media:
        css = {
            'all': [
                '/static/djeym/css/djeym_admin.min.css',
            ]
        }

        js = (
            '/static/djeym/js/jquery-3.3.1.min.js',
            '/static/djeym/js/ckeditor_resize_image.min.js',
        )


@admin.register(HeatPoint)
class HeatPointAdmin(admin.ModelAdmin):
    # ckeditor_change_form.html - Used by default.
    change_form_template = 'djeym/admin/ckeditor_change_form.html'
    list_display = ('title', 'weight', 'slug', 'active')
    list_editable = ('active',)
    list_filter = ('ymap',)
    readonly_fields = ('coordinates', 'slug')
    search_fields = ('title',)

    class Media:
        css = {
            'all': [
                '/static/djeym/css/djeym_admin.min.css',
            ]
        }


@admin.register(CustomClusterIcon)
class CustomClusterIconAdmin(admin.ModelAdmin):
    # ckeditor_change_form.html - Used by default.
    change_form_template = 'djeym/admin/ckeditor_change_form.html'
    list_display = ('title', 'admin_thumbnail')
    list_display_links = ('title', 'admin_thumbnail')
    readonly_fields = ('size_width', 'size_height', 'offset_x', 'offset_y')

    formfield_overrides = {
        models.FileField: {'widget': AdminFileThumbWidget()},
    }

    class Media:
        css = {
            'all': [
                '/static/djeym/css/djeym_admin.min.css',
            ]
        }

        js = [
            '/static/djeym/js/jquery-3.3.1.min.js',
            '/static/djeym/js/get_icon_name.min.js'
        ]


@admin.register(IconCollection)
class IconCollectionAdmin(admin.ModelAdmin):
    # ckeditor_change_form.html - Used by default.
    change_form_template = 'djeym/admin/ckeditor_change_form.html'
    change_list_template = 'djeym/admin/icon_collection_change_list.html'
    list_display = ('title', 'admin_thumbnail', 'get_export_file_btn',
                    'get_icon_count', 'get_count_of_active_icons')
    list_display_links = ('title', 'admin_thumbnail')
    readonly_fields = ('slug',)

    class Media:
        css = {
            'all': [
                '/static/djeym/css/djeym_admin.min.css',
            ]
        }

        js = [
            '/static/djeym/js/jquery-3.3.1.min.js',
            '/static/djeym/js/import_export.min.js'
        ]


@admin.register(CustomMarkerIcon)
class CustomMarkerIconAdmin(admin.ModelAdmin):
    form = OffsetCustomIconForm
    change_form_template = 'djeym/admin/check_icon_offset_change_form.html'
    list_display = ('title', 'admin_thumbnail', 'get_collection_name',
                    'active', 'slug')
    list_display_links = ('title', 'admin_thumbnail')
    readonly_fields = ('size_width', 'size_height', 'slug')
    list_editable = ('active',)
    list_filter = ('icon_collection',)

    formfield_overrides = {
        models.FileField: {'widget': AdminFileThumbWidget()},
    }

    class Media:
        css = {
            'all': [
                '/static/djeym/css/djeym_admin.min.css',
            ]
        }

        js = [
            '/static/djeym/js/jquery-3.3.1.min.js',
            '/static/djeym/plugins/jquery_mousewheel/jquery.mousewheel.min.js',
            '/static/djeym/js/get_icon_name.min.js'
        ]


@admin.register(LoadIndicator)
class LoadIndicatorAdmin(admin.ModelAdmin):
    # ckeditor_change_form.html - Used by default.
    change_form_template = 'djeym/admin/ckeditor_change_form.html'

    list_display = ('title', 'admin_thumbnail', 'slug')
    list_display_links = ('title', 'admin_thumbnail')
    readonly_fields = ('slug',)

    formfield_overrides = {
        models.FileField: {'widget': AdminFileThumbWidget()},
    }

    class Media:
        css = {
            'all': [
                '/static/djeym/css/djeym_admin.min.css',
            ]
        }

        js = [
            '/static/djeym/js/jquery-3.3.1.min.js',
            '/static/djeym/js/get_icon_name.min.js'
        ]



# Generated by Django 2.1.7 on 2019-03-21 09:53

from django.db import migrations, models
import django.db.models.deletion
import djeym.utils


class Migration(migrations.Migration):

    dependencies = [
        ('djeym', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='LoadIndicator',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('svg', models.FileField(help_text='https://github.com/genkosta/django-editor-ymaps/blob/master/Spinner.zip', null=True, upload_to=djeym.utils.make_upload_path, validators=[djeym.utils.validate_svg], verbose_name='Icon')),
                ('title', models.CharField(default='', max_length=60, unique=True, verbose_name='Title')),
                ('slug', models.SlugField(blank=True, max_length=255, null=True)),
            ],
            options={
                'verbose_name': 'Load indicator',
                'verbose_name_plural': 'Load indicators',
                'ordering': ('title', 'id'),
            },
        ),
        migrations.AddField(
            model_name='map',
            name='load_indicator_size',
            field=models.PositiveSmallIntegerField(choices=[(64, '64'), (96, '96'), (128, '128')], default=64, verbose_name='Load indicator size'),
        ),
        migrations.AddField(
            model_name='map',
            name='load_indicator',
            field=models.ForeignKey(blank=True, help_text='https://github.com/genkosta/django-editor-ymaps/blob/master/Spinner.zip', null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='map_load_indicator', to='djeym.LoadIndicator', verbose_name='Load indicator'),
        ),
    ]



# -*- coding: utf-8 -*-
import re

from django import template
from django.conf import settings
from django.utils.safestring import mark_safe

from djeym.models import CustomMarkerIcon, LoadIndicator, Map, TileSource

register = template.Library()


@register.inclusion_tag('djeym/includes/loadymap.html', takes_context=True)
def djeym_load_ymap(context, slug='', panel='djeym/includes/panel.html'):
    """Load YMap"""
    ymap = Map.objects.filter(slug=slug, active=True).first()
    result = {'ymap': ymap}

    if ymap is not None:
        result.update({
            'language_code': context['request'].LANGUAGE_CODE,
            'cluster': ymap.icon_cluster,
            'tile': ymap.tile,
            'presets': ymap.presets.all(),
            'controls': ymap.controls,
            'external_modules': ymap.external_modules,
            'heatmap_settings': ymap.heatmap_settings,
            'general_settings': ymap.general_settings,
            'load_indicators': LoadIndicator.objects.all(),
            'selected_load_indicator': ymap.load_indicator,
            'load_indicator_size': ymap.load_indicator_size
        })

        if not ymap.general_settings.disable_site_panel:
            result.update({
                'category_placemarks': ymap.category_placemark.filter(active=True),
                'category_submarks': ymap.subcategory_placemark.filter(active=True),
                'category_polylines': ymap.category_polyline.filter(active=True),
                'category_polygons': ymap.category_polygon.filter(active=True),
                'panel_path': panel
            })

    return result


@register.inclusion_tag('djeym/includes/api_and_plugins.html', takes_context=True)
def get_api_ymap(context, lang="", ns='djeymYMaps'):
    """Get API for Yandex map"""

    api_version = '2.1'
    api_key = getattr(settings, 'DJEYM_YMAPS_API_KEY', "")
    mode = getattr(settings, 'DJEYM_YMAPS_DOWNLOAD_MODE', 'release')
    lang = lang[:2].lower() if len(lang) > 0 else \
        getattr(settings, 'LANGUAGE_CODE', "en")[:2].lower()

    if lang == 'ru':
        lang += '_RU'
    elif lang == 'en':
        lang += '_US'
    elif lang == 'uk':
        lang += '_UA'
    elif lang == 'tr':
        lang += '_TR'
    else:
        lang = 'en_US'

    return {
        'api_key': api_key,
        'api_version': api_version,
        'lang': lang,
        'mode': mode,
        'ns': ns,
        'external_modules': context.get('external_modules'),
        'presets': context.get('presets')
    }


@register.simple_tag
def random_domain(value, apikey=""):
    """Tile Sources - Add random selection of subdomains, api key"""
    count_elem = re.search(r'\[\[(.+)\]\]', value)

    if count_elem is not None:
        count_elem = len(eval(count_elem.group(0))[0])

    value = re.sub(r'\[(\[.+\])\]',
                   '" + \\1[ Math.round( Math.random() * {} ) ] + "'
                   .format(int(count_elem) - 1), value)
    if len(apikey) > 0:
        var_key = re.search(r'\{\{(.+)\}\}', value).group(0)
        clean_var_key = re.sub(r'{{|}}', "", var_key)
        value = value.replace(var_key, clean_var_key + apikey, 1)
    else:
        value = re.sub(r'\{\{.+\}\}', "", value)
    return mark_safe(value)


@register.inclusion_tag('djeym/includes/geocoder.html', takes_context=True)
def ymap_geocoder(context, address="", controls="zoom", tile_slug='default',
                  marker_slug='default', load_indicator_slug='default', size='64', speed='0.8'):

    load_indicator = LoadIndicator.objects.filter(
        slug=load_indicator_slug).first()

    result = {
        'language_code': context['request'].LANGUAGE_CODE,
        'address': address,
        'controls': True if controls != 'all' else False,
        'tile': TileSource.objects.filter(slug=tile_slug).first(),
        'marker': CustomMarkerIcon.objects.filter(slug=marker_slug).first(),
        'load_indicator': load_indicator.svg.url if
        load_indicator is not None else '/static/djeym/img/spinner.svg',
        'load_indicator_size': size,
        'speed': speed
    }

    return result



# Generated by Django 2.1.7 on 2019-03-22 07:58

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('djeym', '0002_auto_20190321_0953'),
    ]

    operations = [
        migrations.AddField(
            model_name='map',
            name='disable_indicator_animation',
            field=models.BooleanField(default=False, verbose_name='Disable loading indicator animation'),
        ),
    ]



# -*- coding: utf-8 -*-
from ckeditor_uploader.widgets import CKEditorUploadingWidget
from django import forms
from django.utils.translation import ugettext_lazy as _

from .models import (CustomMarkerIcon, GeneralSettings, HeatmapSettings,
                     HeatPoint, Map, MapControls, Placemark, Polygon, Polyline,
                     Preset)
from .widgets import CenterMapWidget, CheckIconOffsetWidget


class GeneralSettingsForm(forms.ModelForm):

    class Meta:
        model = GeneralSettings
        fields = ('ymap', 'clustering_edit', 'clustering_site',
                  'cluster_layout', 'cluster_icon_content',
                  'cluster_icon_content_bg_color', 'cluster_icon_content_txt_color')


class PresetForm(forms.ModelForm):

    class Meta:
        model = Preset
        fields = ('ymap', 'autoheader', 'autobody', 'autofooter',
                  'placemark', 'polyline', 'polygon', 'position')


class HeatPointForm(forms.ModelForm):

    class Meta:
        model = HeatPoint
        fields = ('ymap', 'title', 'weight', 'coordinates')


class HeatmapSettingsForm(forms.ModelForm):

    class Meta:
        model = HeatmapSettings
        fields = ('ymap', 'radius', 'dissipating', 'opacity', 'intensity',
                  'gradient_color1', 'gradient_color2', 'gradient_color3',
                  'gradient_color4')

        widgets = {
            'ymap': forms.HiddenInput(attrs={'id': 'id_djeym_heatmap_ymap'}),
            'radius': forms.NumberInput(attrs={'id': 'id_djeym_heatmap_radius'}),
            'dissipating': forms.CheckboxInput(attrs={'id': 'id_djeym_heatmap_dissipating'}),
            'opacity': forms.TextInput(attrs={'id': 'id_djeym_heatmap_opacity'}),
            'intensity': forms.TextInput(attrs={'id': 'id_djeym_heatmap_intensity'}),
            'gradient_color1': forms.TextInput(
                attrs={'id': 'id_djeym_heatmap_gradient_color1',
                       'class': 'djeym_heatmap_gradient_color'}),
            'gradient_color2': forms.TextInput(
                attrs={'id': 'id_djeym_heatmap_gradient_color2',
                       'class': 'djeym_heatmap_gradient_color'}),
            'gradient_color3': forms.TextInput(
                attrs={'id': 'id_djeym_heatmap_gradient_color3',
                       'class': 'djeym_heatmap_gradient_color'}),
            'gradient_color4': forms.TextInput(
                attrs={'id': 'id_djeym_heatmap_gradient_color4',
                       'class': 'djeym_heatmap_gradient_color'})
        }


class MapControlsForm(forms.ModelForm):

    class Meta:
        model = MapControls
        fields = ('ymap', 'geolocation', 'search', 'provider',
                  'route', 'traffic', 'typeselector', 'fullscreen',
                  'zoom', 'ruler', 'maptype')

        widgets = {
            'ymap': forms.HiddenInput(attrs={'id': 'id_djeym_controls_ymap'}),
            'geolocation': forms.CheckboxInput(attrs={'id': 'id_djeym_controls_geolocation'}),
            'search': forms.CheckboxInput(attrs={'id': 'id_djeym_controls_search'}),
            'provider': forms.CheckboxInput(attrs={'id': 'id_djeym_controls_provider'}),
            'route': forms.CheckboxInput(attrs={'id': 'id_djeym_controls_route'}),
            'traffic': forms.CheckboxInput(attrs={'id': 'id_djeym_controls_traffic'}),
            'typeselector': forms.CheckboxInput(attrs={'id': 'id_djeym_controls_typeselector'}),
            'fullscreen': forms.CheckboxInput(attrs={'id': 'id_djeym_controls_fullscreen'}),
            'zoom': forms.CheckboxInput(attrs={'id': 'id_djeym_controls_zoom'}),
            'ruler': forms.CheckboxInput(attrs={'id': 'id_djeym_controls_ruler'}),
        }


class PlacemarkForm(forms.ModelForm):

    class Meta:
        model = Placemark
        fields = ('ymap', 'category', 'subcategories', 'header',
                  'body', 'footer', 'icon_name', 'coordinates')


class PolylineForm(forms.ModelForm):

    class Meta:
        model = Polyline
        fields = ('ymap', 'category', 'header', 'body', 'footer',
                  'stroke_width', 'stroke_color', 'stroke_opacity', 'coordinates')

        widgets = {
            'stroke_opacity': forms.TextInput()
        }


class PolygonForm(forms.ModelForm):

    class Meta:
        model = Polygon
        fields = ('ymap', 'category', 'header', 'body', 'footer',
                  'stroke_width', 'stroke_color', 'stroke_opacity', 'fill_color',
                  'fill_opacity', 'coordinates')

        widgets = {
            'stroke_opacity': forms.TextInput(),
            'fill_opacity': forms.TextInput(),
        }


class GeoObjectsTransferForm(forms.Form):

    def __init__(self, ymap_id, *args, **kwargs):
        super(GeoObjectsTransferForm, self).__init__(*args, **kwargs)
        self.fields['ymap'].initial = ymap_id

    ymap = forms.CharField(
        widget=forms.TextInput(),
        required=False
    )
    category = forms.CharField(
        widget=forms.TextInput(),
        required=False
    )
    subcategories = forms.CharField(
        widget=forms.SelectMultiple(),
        required=False
    )
    header = forms.CharField(
        widget=forms.Textarea(),
        required=False
    )
    body = forms.CharField(
        widget=forms.Textarea(),
        required=False
    )
    footer = forms.CharField(
        widget=forms.Textarea(),
        required=False
    )
    icon_name = forms.CharField(
        widget=forms.TextInput(),
        required=False
    )
    coordinates = forms.CharField(
        widget=forms.Textarea(),
        required=False
    )
    stroke_width = forms.CharField(
        widget=forms.TextInput(),
        required=False
    )
    stroke_color = forms.CharField(
        widget=forms.TextInput(),
        required=False
    )
    stroke_opacity = forms.CharField(
        widget=forms.TextInput(),
        required=False
    )
    fill_color = forms.CharField(
        widget=forms.TextInput(),
        required=False
    )
    fill_opacity = forms.CharField(
        widget=forms.TextInput(),
        required=False
    )
    pk = forms.CharField(
        widget=forms.TextInput(),
        required=False
    )
    geo_type = forms.CharField(
        widget=forms.TextInput(),
        required=False
    )
    action = forms.CharField(
        widget=forms.TextInput(),
        required=False
    )

    title = forms.CharField(
        widget=forms.TextInput(),
        required=False
    )

    weight = forms.CharField(
        widget=forms.TextInput(),
        required=False
    )


class CenterMapForm(forms.ModelForm):

    class Meta:
        model = Map
        fields = '__all__'

    center_map = forms.CharField(
        widget=CenterMapWidget,
        label=_('Center map'),
        required=False
    )


class OffsetCustomIconForm(forms.ModelForm):

    class Meta:
        model = CustomMarkerIcon
        fields = '__all__'

    check_icon_offset = forms.CharField(
        widget=CheckIconOffsetWidget,
        label=_('Check icon offset'),
        required=False
    )


class CKEditorTextareaForm(forms.Form):

    ckeditor_textarea = forms.CharField(
        widget=CKEditorUploadingWidget(config_name='djeym'),
    )



# -*- coding: utf-8 -*-
import json
from decimal import Decimal

from django.core.exceptions import ValidationError
from django.db import models
from django.db.models.signals import (m2m_changed, post_save, pre_delete,
                                      pre_save)
from django.urls import reverse
from django.utils import timezone
from django.utils.safestring import mark_safe
from django.utils.translation import ugettext_lazy as _
from django.utils.translation import ungettext_lazy
from imagekit.models import ImageSpecField
from pilkit.processors import ResizeToFill
from slugify import slugify
from smart_selects.db_fields import ChainedForeignKey, ChainedManyToManyField

from .raw_presets import raw_presets
from .utils import (DJEYM_YMAPS_ICONS_MAX_SIZE, cleaning_files_pre_delete,
                    cleaning_files_pre_save, get_size_correction,
                    get_size_from_svg, make_upload_path, validate_coordinates,
                    validate_hex_color, validate_image, validate_svg,
                    validate_transparency)

ZOOM_CHOICES = [(num, num) for num in range(24)]

# Load indicator animation speed
ANIMATION_SPEED = (
    ('0.3', '0.3'),
    ('0.4', '0.4'),
    ('0.5', '0.5'),
    ('0.6', '0.6'),
    ('0.7', '0.7'),
    ('0.8', '0.8'),
    ('0.9', '0.9'),
    ('1', '1'),
    ('1.1', '1.1'),
    ('1.2', '1.2'),
    ('1.3', '1.3'),
    ('1.4', '1.4'),
    ('1.5', '1.5'),
    ('1.6', '1.6'),
    ('1.7', '1.7'),
    ('1.8', '1.8'),
)

TRANSPARENCY_CHOICES = (
    ('0', '0'),
    ('0.1', '0.1'),
    ('0.2', '0.2'),
    ('0.3', '0.3'),
    ('0.4', '0.4'),
    ('0.5', '0.5'),
    ('0.6', '0.6'),
    ('0.7', '0.7'),
    ('0.8', '0.8'),
    ('0.9', '0.9'),
    ('1', '1')
)

CLUSTER_BALLOON_CONTENT_LAYOUT_CHOICES = (
    ('cluster#balloonTwoColumns', _('Two Columns')),
    ('cluster#balloonCarousel', _('Carousel'))
)

MAP_TYPE_CHOICES = (
    ('yandex#map', _('Scheme')),
    ('yandex#satellite', _('Satellite')),
    ('yandex#hybrid', _('Hybrid'))
)

LOAD_INDICATOR_SIZE_CHOICES = (
    (64, "64"),
    (96, "96"),
    (128, "128")
)

FEATURE_POINT = {
    "type": "Feature",
    "id": 0,
    "geometry": {
        "type": "Point",
        "coordinates": []
    },
    "properties": {
        "balloonContentHeader": "",
        "balloonContentBody": "",
        "balloonContentFooter": "",
        "id": 0,
        "categoryID": 0,
        "subCategoryIDs": [],
        "iconName": ""
    },
    "options": {
        "iconLayout": "default#image",
        "iconImageHref": "",
        "iconImageSize": [],
        "iconImageOffset": []
    }
}

FEATURE_LINE = {
    "type": "Feature",
    "id": 0,
    "geometry": {
        "type": "LineString",
        "coordinates": []
    },
    "properties": {
        "balloonContentHeader": "",
        "balloonContentBody": "",
        "balloonContentFooter": "",
        "id": 0,
        "categoryID": 0
    },
    "options": {
        "strokeWidth": 5,
        "strokeColor": "#1e98ff",
        "strokeOpacity": 0.9
    }
}

FEATURE_POLYGON = {
    "type": "Feature",
    "id": 0,
    "geometry": {
        "type": "Polygon",
        "coordinates": []
    },
    "properties": {
        "balloonContentHeader": "",
        "balloonContentBody": "",
        "balloonContentFooter": "",
        "id": 0,
        "categoryID": 0
    },
    "options": {
        "strokeWidth": 2,
        "strokeColor": "#1e98ff",
        "strokeOpacity": 0.9,
        "fillColor": "#1e98ff",
        "fillOpacity": 0.9
    }
}

FEATURE_HEAT_POINT = {
    "type": "Feature",
    "id": 0,
    "geometry": {
        "type": "Point",
        "coordinates": []
    },
    "properties": {
        "weight": 0
    }
}


class TileSource(models.Model):
    """Source tile layer"""

    title = models.CharField(
        _('Title'), max_length=255, unique=True, default="")

    maxzoom = models.PositiveSmallIntegerField(
        _('Maximum zoom'), choices=ZOOM_CHOICES, default=12)

    minzoom = models.PositiveSmallIntegerField(
        _('Minimum zoom'), choices=ZOOM_CHOICES, default=0)

    source = models.TextField(_('Source'), default="")

    screenshot = models.ImageField(
        _('Screenshot'),
        upload_to=make_upload_path,
        validators=[validate_image],
        null=True,
        help_text=_('Recommended size - Width=360 x Height=180'))

    copyrights = models.TextField(
        _('Copyrights'), blank=True, default="")

    site = models.URLField(_('Site'), blank=True, default="")

    apikey = models.CharField(
        _('API Key'),
        max_length=255,
        blank=True,
        default="",
        help_text=_('API key or access_token'))

    note = models.TextField(_('Note'), blank=True, default="")
    slug = models.SlugField(unique=True, max_length=255, blank=True, null=True)

    middle = ImageSpecField(
        [ResizeToFill(360, 180, upscale=True)], source='screenshot')

    @property
    def upload_dir(self):
        return 'djeym_tile_screenshot'

    def __str__(self):
        return '{}'.format(self.title)

    class Meta:
        ordering = ('title',)
        verbose_name = _('Tile Source')
        verbose_name_plural = _('Tile Sources')

    def save(self, *args, **kwargs):
        self.slug = slugify(self.title)
        super(TileSource, self).save(*args, **kwargs)

    def admin_thumbnail(self):
        if bool(self.screenshot):
            return mark_safe('<img src="{0}" height="{1}" alt="Screenshot">'
                             .format(self.screenshot.url, 60))
        else:
            return ""
    admin_thumbnail.short_description = _('Screenshot')


class MapControls(models.Model):
    """Map Controls"""

    ymap = models.OneToOneField(
        'Map',
        verbose_name=ungettext_lazy('Map', 'Map', 1),
        related_name='controls',
        null=True,
        on_delete=models.CASCADE
    )

    geolocation = models.BooleanField(_('Geolocation'), default=True)
    search = models.BooleanField(_('Search by map'), default=True)
    provider = models.BooleanField(
        _('Search by organization name'), default=True)
    route = models.BooleanField(_('Routing panel'), default=True)
    traffic = models.BooleanField(_('Traffic jams'), default=True)
    typeselector = models.BooleanField(_('Map Layer Switch'), default=True)
    fullscreen = models.BooleanField(_('Full screen mode'), default=True)
    zoom = models.BooleanField(_('Zoom'), default=True)
    ruler = models.BooleanField(_('Ruler'), default=True)
    maptype = models.CharField(
        _('Map type'),
        max_length=255,
        choices=MAP_TYPE_CHOICES,
        default='yandex#map')

    def __str__(self):
        return '{}'.format(_('Map Controls'))

    class Meta:
        verbose_name_plural = _('Map Controls')

    def get_control_list(self):
        """Get a list of selected controls."""
        controls_dict = {
            'geolocation': 'geolocationControl',
            'search': 'searchControl',
            'route': 'routeButtonControl',
            'traffic': 'trafficControl',
            'typeselector': 'typeSelector',
            'fullscreen': 'fullscreenControl',
            'zoom': 'zoomControl',
            'ruler': 'rulerControl'
        }
        return repr([val for key, val in controls_dict.items() if getattr(self, key)])


class ExternalModules(models.Model):
    """External modules - Plugins"""

    ymap = models.OneToOneField(
        'Map',
        verbose_name=ungettext_lazy('Map', 'Map', 1),
        related_name='external_modules',
        null=True,
        on_delete=models.CASCADE
    )

    roundtheme = models.BooleanField(
        _('Round map controls theme'), default=False)

    heatmap = models.BooleanField(ungettext_lazy(
        'Heatmap', 'Heat maps', 1), default=False)

    areacalculation = models.BooleanField(
        _('Area Calculation'), default=True)

    def __str__(self):
        return '{}'.format(_('Enable'))

    class Meta:
        verbose_name = _('External modules')
        verbose_name_plural = _('External modules')


class HeatmapSettings(models.Model):
    """Heatmap settings"""

    ymap = models.OneToOneField(
        'Map',
        verbose_name=ungettext_lazy('Map', 'Map', 1),
        related_name='heatmap_settings',
        null=True,
        on_delete=models.CASCADE
    )

    radius = models.PositiveIntegerField(
        _('Point radius of influence (px)'),
        default=10)

    dissipating = models.BooleanField(
        _('Disperse points'),
        default=False,
        help_text=_("Enable - Disperse points on higher zoom levels according to radius. "
                    "Disable - Don't disperse."))

    opacity = models.CharField(
        _('Transparency of heat layer'),
        max_length=255,
        choices=TRANSPARENCY_CHOICES,
        validators=[validate_transparency],
        default='0.8')

    intensity = models.CharField(
        _('Intensity of median point'),
        max_length=255,
        choices=TRANSPARENCY_CHOICES,
        validators=[validate_transparency],
        default='0.2')

    gradient_color1 = models.CharField(
        'gradient color 1',
        max_length=255,
        default='#56db40b3')

    gradient_color2 = models.CharField(
        'gradient color 2',
        max_length=255,
        default='#ffd21ecc')

    gradient_color3 = models.CharField(
        'gradient color 3',
        max_length=255,
        default='#ed4543e6')

    gradient_color4 = models.CharField(
        'gradient color 4',
        max_length=255,
        default='#b22222')

    def __str__(self):
        return '{}'.format(_('Settings'))

    class Meta:
        verbose_name = ungettext_lazy('Heatmap', 'Heat maps', 1)
        verbose_name_plural = ungettext_lazy('Heatmap', 'Heat maps', 1)


class Preset(models.Model):
    """Preset custom solution"""

    ymap = models.ForeignKey(
        'Map',
        verbose_name=ungettext_lazy('Map', 'Map', 1),
        related_name='presets',
        null=True,
        on_delete=models.CASCADE)

    title = models.CharField(_('Title'), max_length=60, default="")

    icon = models.CharField(
        _('Icon (html tag)'),
        max_length=255,
        blank=True,
        default='<i class="far fa-smile"></i>',
        help_text=_('https://fontawesome.com/icons?d=gallery&m=free - '
                    'Example: &lt;i class="fab fa-android"&gt;&lt;/i&gt;'))

    html = models.TextField(
        'HTML', default='<p style="color:#e91e63;">iPreset</p>')

    js = models.TextField('JS', blank=True, default="",
                          help_text='JavaScript and jQuery')

    description = models.TextField(
        _('Description'),
        default="""<div>Описание</div>
<div style="color:#607D8B;">Description</div>""",
        help_text=_('Brief description of the preset.'))

    autoheader = models.BooleanField(
        _('Automatically add to end of header'), default=False)
    autobody = models.BooleanField(
        _('Automatically add to end of description'), default=False)
    autofooter = models.BooleanField(
        _('Automatically add to end of footer'), default=False)

    placemark = models.BooleanField(_('Markers'), default=True)
    polyline = models.BooleanField(_('Routes'), default=True)
    polygon = models.BooleanField(ungettext_lazy(
        'Territory', 'Territorys', 2), default=True)

    position = models.PositiveSmallIntegerField(
        _('Position'), default=0, blank=True)

    slug = models.SlugField(max_length=255, blank=True, null=True)

    def __str__(self):
        return '{}'.format(self.title)

    class Meta:
        ordering = ('position',)
        verbose_name = _('Preset')
        verbose_name_plural = _('Presets')

    def save(self, *args, **kwargs):
        self.slug = slugify(self.title)
        super(Preset, self).save(*args, **kwargs)

    def clean(self):
        ymap = self.ymap
        raw_slug = slugify(self.title)
        if not bool(self.pk) and Preset.objects.filter(ymap=ymap, slug=raw_slug).count() > 0:
            raise ValidationError(
                {'title': _('A preset with this name already exists for this map.')})


class GeneralSettings(models.Model):
    """General map settings"""

    ymap = models.OneToOneField(
        'Map',
        verbose_name=ungettext_lazy('Map', 'Map', 1),
        related_name='general_settings',
        null=True,
        on_delete=models.CASCADE
    )

    clustering_edit = models.BooleanField(
        _('Сlustering in editor'), default=True)

    clustering_site = models.BooleanField(
        _('Сlustering on site'), default=True)

    cluster_layout = models.CharField(
        _('Layout for the cluster information window'),
        max_length=255,
        choices=CLUSTER_BALLOON_CONTENT_LAYOUT_CHOICES,
        default='cluster#balloonTwoColumns')

    cluster_icon_content = models.BooleanField(
        _('Display the number of objects in the cluster icon'), default=True)

    cluster_icon_content_bg_color = models.CharField(
        _('Background color'),
        max_length=255,
        default='#ffffff',
        validators=[validate_hex_color])

    cluster_icon_content_txt_color = models.CharField(
        _('Text color'),
        max_length=255,
        default='#333333',
        validators=[validate_hex_color])

    disable_site_panel = models.BooleanField(
        _('Disable the panel on the site (button with the image of '
          'the gear and the eye in the center)'),
        default=False)

    def __str__(self):
        return '{}'.format(_('Settings'))

    class Meta:
        verbose_name = _('General settings')
        verbose_name_plural = _('General settings')


class Map(models.Model):
    """Create a map"""

    title = models.CharField(
        _('Title'),
        max_length=60,
        unique=True,
        help_text=_('Examples: Supermarkets of the city of Kharkov or '
                    'Hawaii - Oahu Island or Luxury hotels in Honolulu or '
                    'Parking of primitive people, etc.'))

    icon_cluster = models.ForeignKey(
        'CustomClusterIcon',
        verbose_name=ungettext_lazy(
            'Icon for cluster', 'Icons for clusters', 1),
        related_name='placemark_custom_cluster',
        null=True,
        on_delete=models.SET_NULL)

    icon_collection = models.ForeignKey(
        'IconCollection',
        verbose_name=ungettext_lazy('Icon collection for markers',
                                    'Icon collections for markers', 1),
        related_name='map_icon_collection',
        null=True,
        on_delete=models.SET_NULL
    )

    tile = models.ForeignKey(
        TileSource,
        verbose_name=_('Tile Source'),
        related_name='maptile',
        blank=True,
        null=True,
        on_delete=models.SET_NULL
    )

    load_indicator = models.ForeignKey(
        'LoadIndicator',
        verbose_name=_('Load indicator'),
        related_name='map_load_indicator',
        blank=True,
        null=True,
        on_delete=models.SET_NULL,
        help_text=_(
            'https://github.com/genkosta/django-editor-ymaps/blob/master/Spinner.zip')
    )

    load_indicator_size = models.PositiveSmallIntegerField(
        _('Load indicator size'),
        choices=LOAD_INDICATOR_SIZE_CHOICES,
        default=64
    )

    animation_speed = models.CharField(
        _('Load indicator animation speed'),
        max_length=255,
        choices=ANIMATION_SPEED,
        default='0.8')

    disable_indicator_animation = models.BooleanField(
        _('Disable loading indicator animation'),
        default=False,
        help_text=_(
            'It may be useful for the abbreviation or logo of the company, '
            'if it does not make sense to animate them.')
    )

    active = models.BooleanField(_('Active map'), default=True)

    zoom = models.PositiveSmallIntegerField(
        _('Zoom'), choices=ZOOM_CHOICES, default=3)

    latitude = models.CharField(_('Latitude'), max_length=255, default='0',
                                validators=[validate_coordinates],
                                help_text=_('Latitude center of the map.'))

    longitude = models.CharField(_('Longitude'), max_length=255, default='0',
                                 validators=[validate_coordinates],
                                 help_text=_('Longitude center of the map.'))

    slug = models.SlugField(unique=True, max_length=255, blank=True, null=True)

    created_at = models.DateTimeField(_('Created at'), auto_now_add=True)
    updated_at = models.DateTimeField(_('Updated at'), auto_now=True)

    def __str__(self):
        return '{}'.format(self.title)

    class Meta:
        ordering = ('title',)
        verbose_name = ungettext_lazy('Map', 'Map', 5)
        verbose_name_plural = ungettext_lazy('Map', 'Map', 2)

    def clean(self):
        """
        We check the presence in the collection of at least one active icon.
        Проверяем наличие в коллекции хотя бы одной активной иконки.
        """
        if bool(self.icon_collection) and \
                self.icon_collection.icons.filter(active=True).count() == 0:
            raise ValidationError({'icon_collection':
                                   _('The collection must have at least one active icon.')})

    @staticmethod
    def create_preset(ymap, raw_preset):
        Preset.objects.create(
            ymap=ymap,
            title=raw_preset['title'],
            icon=raw_preset['icon'],
            html=raw_preset['html'],
            js=raw_preset['js'],
            description=raw_preset['description']
        )

    def save(self, *args, **kwargs):
        self.slug = slugify(self.title)
        super(Map, self).save(*args, **kwargs)
        if not hasattr(self, 'controls'):
            MapControls.objects.create(ymap=self)
        if not hasattr(self, 'external_modules'):
            ExternalModules.objects.create(ymap=self)
        if not hasattr(self, 'heatmap_settings'):
            HeatmapSettings.objects.create(ymap=self)
        if not hasattr(self, 'general_settings'):
            GeneralSettings.objects.create(ymap=self)
        if hasattr(self, 'presets'):
            ymap = Map.objects.get(pk=self.pk)
            slugs = [item.slug for item in ymap.presets.all()]
            if len(slugs) > 0:
                for raw_preset in raw_presets:
                    if raw_preset['slug'] not in slugs:
                        self.create_preset(ymap, raw_preset)
            else:
                for raw_preset in raw_presets:
                    self.create_preset(ymap, raw_preset)

    def get_custom_cluster(self):
        if bool(self.icon_cluster):
            return mark_safe('<img src="{0}" height="{1}" alt="Cluster Icon">'
                             .format(self.icon_cluster.svg.url, 46))
        else:
            return ""
    get_custom_cluster.short_description = _('Cluster Icon')

    def get_custom_marker_icon(self):
        icon = CustomMarkerIcon.objects.filter(
            icon_collection=self.icon_collection).first()
        if icon is not None:
            return mark_safe('<img src="{0}" height="{1}" alt="Icon">'
                             .format(icon.svg.url, 46))
        else:
            return ""
    get_custom_marker_icon.short_description = _(
        'Example icon from collection')

    def get_tile_screenshot(self):
        if bool(self.tile):
            return mark_safe('<img src="{0}" height="{1}" alt="Screenshot">'
                             .format(self.tile.screenshot.url, 46))
        else:
            return mark_safe(
                '<img src="/static/djeym/img/default_tile.png" height="46" alt="Screenshot">')
    get_tile_screenshot.short_description = _('Tile screenshot')

    def get_status_heatmap(self):
        if hasattr(self, 'heatmap_settings') and self.external_modules.heatmap:
            return mark_safe(
                '<img src="/static/djeym/img/red_check.svg/" height="30" alt="Icon">')
        else:
            return mark_safe(
                '<img src="/static/djeym/img/colder_minus.svg/" height="30" alt="Icon">')
    get_status_heatmap.short_description = _('Heat map status')

    def get_load_indicator(self):
        if bool(self.load_indicator):
            return mark_safe('<img src="{0}" height="{1}" alt="Icon">'
                             .format(self.load_indicator.svg.url, 46))
        else:
            return ""
    get_load_indicator.short_description = _('Load indicator')

    def get_absolute_url(self):
        return reverse('djeym:editor_ymap', args=(self.slug,))


class CategoryPlacemark(models.Model):
    """Category of placemark"""

    ymap = models.ForeignKey(
        Map,
        verbose_name=ungettext_lazy('Map', 'Map', 1),
        related_name='category_placemark',
        on_delete=models.CASCADE)

    title = models.CharField(
        _('Title'),
        max_length=60,
        help_text=_('Name of any geographic area - Country Name or '
                    'City Name or District Name or The name of the resort coast, etc.'))

    category_icon = models.CharField(
        _('Category Icon (html tag)'),
        max_length=255,
        blank=True,
        default="",
        help_text=_('https://fontawesome.com/icons?d=gallery&m=free - '
                    'Example: &lt;i class="fab fa-android"&gt;&lt;/i&gt;'))

    category_color = models.CharField(
        _("Category color"),
        max_length=255,
        default='#00bfff',
        validators=[validate_hex_color],
        help_text=_('Applies to Checkboxes and Radio Buttons.'))

    active = models.BooleanField(_('Active category'), default=True)

    def __str__(self):
        return '{}'.format(self.title)

    class Meta:
        ordering = ('title',)
        verbose_name = ungettext_lazy(
            'Category of placemark', 'Category of placemarks', 5)
        verbose_name_plural = ungettext_lazy(
            'Category of placemark', 'Category of placemarks', 2)

    def clean(self):
        count_category = CategoryPlacemark.objects.filter(
            ymap=self.ymap, title=self.title).count()
        if not bool(self.pk) and count_category > 0:
            msg = _(
                'A category with this name already exists for the selected map.')
            raise ValidationError({'title': msg})

    def get_map_name(self):
        return "{}".format(self.ymap.title)
    get_map_name.short_description = ungettext_lazy('Map', 'Map', 1)

    def get_title(self):
        return "{}".format(self.title)
    get_title.short_description = _("Category name")

    def get_category_icon(self):
        if bool(self.category_icon):
            return mark_safe('<div class="category_icon">{}</div>'.format(self.category_icon))
        else:
            return ""
    get_category_icon.short_description = _('Category Icon')

    def get_category_color(self):
        return mark_safe("<div class=\"djeym_category_color\" "
                         "style=\"background:{};\"></div>".format(self.category_color))
    get_category_color.short_description = _("Category color")


class SubCategoryPlacemark(models.Model):
    """Subcategory of placemark"""

    ymap = models.ForeignKey(
        Map,
        verbose_name=ungettext_lazy('Map', 'Map', 1),
        related_name='subcategory_placemark',
        on_delete=models.CASCADE)

    title = models.CharField(
        _('Title'),
        max_length=60,
        help_text=_('Subcategory - This is what complements the categories of markers. '
                    'Examples: Parking or Bicycle Parking or ATM or '
                    'Cafe or Wi-Fi or Playground, etc.'))

    category_icon = models.CharField(
        _('Category Icon (html tag)'),
        max_length=255,
        blank=True,
        default="",
        help_text=_('https://fontawesome.com/icons?d=gallery&m=free - '
                    'Example: &lt;i class="fab fa-android"&gt;&lt;/i&gt;'))

    category_color = models.CharField(
        _("Category color"),
        max_length=255,
        default='#ffcc00',
        validators=[validate_hex_color],
        help_text=_('Applies to Checkboxes and Radio Buttons.'))

    active = models.BooleanField(_('Active category'), default=True)

    def __str__(self):
        return '{}'.format(self.title)

    class Meta:
        ordering = ('title',)
        verbose_name = ungettext_lazy(
            'Subcategory of placemark', 'Subcategory of placemarks', 5)
        verbose_name_plural = ungettext_lazy(
            'Subcategory of placemark', 'Subcategory of placemarks', 2)

    def clean(self):
        count_category = SubCategoryPlacemark.objects.filter(
            ymap=self.ymap, title=self.title).count()
        if not bool(self.pk) and count_category > 0:
            msg = _(
                'A category with this name already exists for the selected map.')
            raise ValidationError({'title': msg})

    def get_map_name(self):
        return "{}".format(self.ymap.title)
    get_map_name.short_description = ungettext_lazy('Map', 'Map', 1)

    def get_title(self):
        return "{}".format(self.title)
    get_title.short_description = _("Category name")

    def get_category_icon(self):
        if bool(self.category_icon):
            return mark_safe('<div class="category_icon">{}</div>'.format(self.category_icon))
        else:
            return ""
    get_category_icon.short_description = _('Icon')

    def get_category_color(self):
        return mark_safe("<div class=\"djeym_category_color\" "
                         "style=\"background:{};\"></div>".format(self.category_color))
    get_category_color.short_description = _("Category color")


class CategoryPolyline(models.Model):
    """Category of route"""

    ymap = models.ForeignKey(
        Map,
        verbose_name=ungettext_lazy('Map', 'Map', 1),
        related_name='category_polyline',
        on_delete=models.CASCADE)

    title = models.CharField(
        _('Title'),
        max_length=60,
        help_text=_('Examples: Tourist routes or Marathon routes etc.'))

    category_icon = models.CharField(
        _('Category Icon (html tag)'),
        max_length=255,
        blank=True,
        default="",
        help_text=_('https://fontawesome.com/icons?d=gallery&m=free - '
                    'Example: &lt;i class="fab fa-android"&gt;&lt;/i&gt;'))

    category_color = models.CharField(
        _("Category color"),
        max_length=255,
        default='#00bfff',
        validators=[validate_hex_color],
        help_text=_('Applies to Checkboxes and Radio Buttons.'))

    active = models.BooleanField(_('Active category'), default=True)

    def __str__(self):
        return '{}'.format(self.title)

    class Meta:
        ordering = ('title',)
        verbose_name = ungettext_lazy(
            'Category of route', 'Category of routes', 5)
        verbose_name_plural = ungettext_lazy(
            'Category of route', 'Category of routes', 2)

    def clean(self):
        count_category = CategoryPolyline.objects.filter(
            ymap=self.ymap, title=self.title).count()
        if not bool(self.pk) and count_category > 0:
            msg = _(
                'A category with this name already exists for the selected map.')
            raise ValidationError({'title': msg})

    def get_map_name(self):
        return "{}".format(self.ymap.title)
    get_map_name.short_description = ungettext_lazy('Map', 'Map', 1)

    def get_title(self):
        return "{}".format(self.title)
    get_title.short_description = _("Category name")

    def get_category_icon(self):
        if bool(self.category_icon):
            return mark_safe('<div class="category_icon">{}</div>'.format(self.category_icon))
        else:
            return ""
    get_category_icon.short_description = _('Category Icon')

    def get_category_color(self):
        return mark_safe("<div class=\"djeym_category_color\" "
                         "style=\"background:{};\"></div>".format(self.category_color))
    get_category_color.short_description = _("Category color")


class CategoryPolygon(models.Model):
    """Category of territory"""

    ymap = models.ForeignKey(
        Map,
        verbose_name=ungettext_lazy('Map', 'Map', 1),
        related_name='category_polygon',
        on_delete=models.CASCADE)

    title = models.CharField(
        _('Title'),
        max_length=60,
        help_text=_('The name of the aggregate of any geographic area. '
                    'Examples: Honduran cities or Hawaiian beaches or '
                    'Residential complexes in Antarctica.'))

    category_icon = models.CharField(
        _('Category Icon (html tag)'),
        max_length=255,
        blank=True,
        default="",
        help_text=_('https://fontawesome.com/icons?d=gallery&m=free - '
                    'Example: &lt;i class="fab fa-android"&gt;&lt;/i&gt;'))

    category_color = models.CharField(
        _("Category color"),
        max_length=255,
        default='#00bfff',
        validators=[validate_hex_color],
        help_text=_('Applies to Checkboxes and Radio Buttons.'))

    active = models.BooleanField(_('Active category'), default=True)

    def __str__(self):
        return '{}'.format(self.title)

    class Meta:
        ordering = ('title',)
        verbose_name = ungettext_lazy(
            'Category of territory', 'Category of territories', 5)
        verbose_name_plural = ungettext_lazy(
            'Category of territory', 'Category of territories', 2)

    def clean(self):
        count_category = CategoryPolygon.objects.filter(
            ymap=self.ymap, title=self.title).count()
        if not bool(self.pk) and count_category > 0:
            msg = _(
                'A category with this name already exists for the selected map.')
            raise ValidationError({'title': msg})

    def get_map_name(self):
        return "{}".format(self.ymap.title)
    get_map_name.short_description = ungettext_lazy('Map', 'Map', 1)

    def get_title(self):
        return "{}".format(self.title)
    get_title.short_description = _("Category name")

    def get_category_icon(self):
        if bool(self.category_icon):
            return mark_safe('<div class="category_icon">{}</div>'.format(self.category_icon))
        else:
            return ""
    get_category_icon.short_description = _('Category Icon')

    def get_category_color(self):
        return mark_safe("<div class=\"djeym_category_color\" "
                         "style=\"background:{};\"></div>".format(self.category_color))
    get_category_color.short_description = _("Category color")


class Placemark(models.Model):
    """Placemark"""

    ymap = models.ForeignKey(
        Map,
        verbose_name=ungettext_lazy('Map', 'Map', 1),
        related_name='placemark_map',
        on_delete=models.CASCADE)

    category = ChainedForeignKey(
        CategoryPlacemark,
        verbose_name=_('Category'),
        related_name='placemark_category',
        chained_field="ymap",
        chained_model_field="ymap",
        show_all=False,
        auto_choose=True,
        on_delete=models.CASCADE)

    subcategories = ChainedManyToManyField(
        SubCategoryPlacemark,
        verbose_name=_('Subcategories'),
        related_name='placemark_subcategories',
        chained_field="ymap",
        chained_model_field="ymap",
        blank=True,
        auto_choose=False)

    header = models.TextField(_('Place name'), default="")
    body = models.TextField(_('Content'), blank=True, default="")
    footer = models.TextField(_('Footer'), blank=True, default="")
    icon_name = models.CharField(
        _('Icon name'), max_length=255, default="")

    coordinates = models.CharField(
        _('Coordinates'), max_length=255, default='[0,0]')

    like = models.PositiveIntegerField('Like', default=0, blank=True)
    dislike = models.PositiveIntegerField('Dislike', default=0, blank=True)
    active = models.BooleanField(_('Active placemark'), default=True)

    json_code = models.TextField(
        _('JSON'), blank=True, default=json.dumps(FEATURE_POINT), editable=False)

    created_at = models.DateTimeField(_('Created at'), auto_now_add=True)
    updated_at = models.DateTimeField(_('Updated at'), auto_now=True)

    def __str__(self):
        return mark_safe('{}'.format(self.header))

    class Meta:
        ordering = ("-id",)
        verbose_name = _('Placemark')
        verbose_name_plural = _('Placemarks')


class Polyline(models.Model):
    """Polyline"""

    ymap = models.ForeignKey(
        Map,
        verbose_name=ungettext_lazy('Map', 'Map', 1),
        related_name='polyline_map',
        on_delete=models.CASCADE)

    category = ChainedForeignKey(
        CategoryPolyline,
        verbose_name=_('Category'),
        related_name='polyline_category',
        chained_field="ymap",
        chained_model_field="ymap",
        show_all=False,
        auto_choose=True,
        on_delete=models.CASCADE)

    header = models.TextField(_('Route name'), default="")
    body = models.TextField(_('Content'), blank=True, default="")
    footer = models.TextField(_('Footer'), blank=True, default="")
    stroke_width = models.PositiveIntegerField(_('Stroke width'), default=5)

    stroke_color = models.CharField(
        _('Line color'),
        max_length=255,
        default='#1e98ff',
        validators=[validate_hex_color])

    stroke_opacity = models.CharField(
        _('Opacity line'),
        max_length=255,
        choices=TRANSPARENCY_CHOICES,
        validators=[validate_transparency],
        default='0.9')

    coordinates = models.TextField(_('Coordinates'), default="")
    like = models.PositiveIntegerField('Like', default=0, blank=True)
    dislike = models.PositiveIntegerField('Dislike', default=0, blank=True)
    active = models.BooleanField(_('Active route'), default=True)

    json_code = models.TextField(
        _('JSON'), blank=True, default=json.dumps(FEATURE_LINE), editable=False)

    created_at = models.DateTimeField(_('Created at'), auto_now_add=True)
    updated_at = models.DateTimeField(_('Updated at'), auto_now=True)

    def __str__(self):
        return mark_safe('{}'.format(self.header))

    class Meta:
        ordering = ("-id",)
        verbose_name = _('Route')
        verbose_name_plural = _('Routes')


class Polygon(models.Model):
    """Polygon"""

    ymap = models.ForeignKey(
        Map,
        verbose_name=ungettext_lazy('Map', 'Map', 1),
        related_name='polygon_map',
        on_delete=models.CASCADE)

    category = ChainedForeignKey(
        CategoryPolygon,
        verbose_name=_('Category'),
        related_name='polygon_category',
        chained_field="ymap",
        chained_model_field="ymap",
        show_all=False,
        auto_choose=True,
        on_delete=models.CASCADE)

    header = models.TextField(_('Territory name'), default='')
    body = models.TextField(_('Content'), blank=True, default="",)
    footer = models.TextField(_('Footer'), blank=True, default="")
    stroke_width = models.PositiveIntegerField(_('Stroke width'), default=2)

    stroke_color = models.CharField(
        _('Line color'),
        max_length=255,
        default='#1e98ff',
        validators=[validate_hex_color])

    stroke_opacity = models.CharField(
        _('Opacity line'),
        max_length=255,
        choices=TRANSPARENCY_CHOICES,
        validators=[validate_transparency],
        default='0.9')

    fill_color = models.CharField(
        _('Fill color'),
        max_length=255,
        default='#1e98ff',
        validators=[validate_hex_color])

    fill_opacity = models.CharField(
        _('Fill opacity'),
        max_length=255,
        choices=TRANSPARENCY_CHOICES,
        validators=[validate_transparency],
        default='0.9')

    coordinates = models.TextField(_('Coordinates'), default="")
    like = models.PositiveIntegerField('Like', default=0, blank=True)
    dislike = models.PositiveIntegerField('Dislike', default=0, blank=True)
    active = models.BooleanField(_('Active territory'), default=True)

    json_code = models.TextField(
        _('JSON'), blank=True, default=json.dumps(FEATURE_POLYGON), editable=False)

    created_at = models.DateTimeField(_('Created at'), auto_now_add=True)
    updated_at = models.DateTimeField(_('Updated at'), auto_now=True)

    def __str__(self):
        return mark_safe('{}'.format(self.header))

    class Meta:
        ordering = ("-id",)
        verbose_name = ungettext_lazy('Territory', 'Territorys', 5)
        verbose_name_plural = ungettext_lazy('Territory', 'Territorys', 2)


class HeatPoint(models.Model):
    """Heat Point"""

    ymap = models.ForeignKey(
        Map,
        verbose_name=ungettext_lazy('Map', 'Map', 1),
        related_name='heat_point',
        on_delete=models.CASCADE)

    title = models.CharField(
        _('Place name'),
        max_length=60,
        blank=True,
        default="")

    weight = models.PositiveIntegerField(
        _('Weight'),
        default=0,
        blank=True,
        help_text=_('Examples: Average property value on a district site or '
                    'Number of cameras installed on a building, etc.'))

    coordinates = models.CharField(
        _('Coordinates'), max_length=255, default='[0,0]')

    active = models.BooleanField(_('Active heat point'), default=True)

    slug = models.SlugField(unique=True, max_length=255, blank=True, null=True)

    json_code = models.TextField(
        _('JSON'), blank=True, default=json.dumps(FEATURE_HEAT_POINT), editable=False)

    def __str__(self):
        return '{}'.format(self.title)

    class Meta:
        verbose_name = ungettext_lazy('Heat Point', 'Thermal points', 5)
        verbose_name_plural = ungettext_lazy('Heat Point', 'Thermal points', 2)

    def save(self, *args, **kwargs):
        self.slug = slugify(self.title)
        super(HeatPoint, self).save(*args, **kwargs)
        if len(self.title) == 0:
            self.title = '{0} - {1}'.format(ungettext_lazy(
                'Heat Point', 'Thermal points', 1), self.pk)
            self.save()


class CustomClusterIcon(models.Model):
    """Custom Cluster Icons"""

    svg = models.FileField(
        _('Icon'),
        upload_to=make_upload_path,
        validators=[validate_svg],
        null=True,
        help_text=_('Only SVG files.'))

    title = models.CharField(
        _('Cluster name'),
        max_length=60,
        default="",
        unique=True,
        help_text=_('Example: Green Garden'))

    size_width = models.PositiveSmallIntegerField(_('Icon width'), default=0)
    size_height = models.PositiveSmallIntegerField(_('Icon height'), default=0)

    offset_x = models.DecimalField(
        _('Offset by axis - X'),
        max_digits=3,
        decimal_places=1,
        default=Decimal(".0"))

    offset_y = models.DecimalField(
        _('Offset by axis - Y'),
        max_digits=3,
        decimal_places=1,
        default=Decimal(".0"))

    @property
    def upload_dir(self):
        return 'djeym_custom_icons'

    def __str__(self):
        return '{}'.format(self.title)

    def admin_thumbnail(self):
        if bool(self.svg):
            return mark_safe('<img src="{0}" height="{1}" alt="Icon">'
                             .format(self.svg.url, DJEYM_YMAPS_ICONS_MAX_SIZE))
        else:
            return ""
    admin_thumbnail.short_description = _('Icon')

    class Meta:
        ordering = ("title", "id")
        verbose_name = ungettext_lazy(
            'Icon for cluster', 'Icons for clusters', 5)
        verbose_name_plural = ungettext_lazy(
            'Icon for cluster', 'Icons for clusters', 2)

    def __init__(self, *args, **kwargs):
        super(CustomClusterIcon, self).__init__(*args, **kwargs)
        self.__old_image = self.svg

    def save(self, *args, **kwargs):
        self.slug = slugify("{}".format(self.title))
        old_image = self.__old_image
        new_image = self.svg
        if not bool(old_image) or (bool(old_image) and bool(new_image) and
                                   old_image.name != new_image.name):
            self.__old_image = new_image
            self.size_width = 0
            self.size_height = 0
            self.offset_x = Decimal(".0")
            self.offset_y = Decimal(".0")
        super(CustomClusterIcon, self).save(*args, **kwargs)

    def get_size(self):
        return '[{0},{1}]'.format(self.size_width, self.size_height)

    def get_offset(self):
        return '[{0:f},{1:f}]'.format(self.offset_x, self.offset_y)


class IconCollection(models.Model):
    """Icon collection"""

    title = models.CharField(
        _('Collection name'),
        max_length=60,
        default="",
        unique=True,
        help_text=_('Example: Light golden Avocado'))

    slug = models.SlugField(unique=True, max_length=255, blank=True, null=True)

    def __str__(self):
        return '{}'.format(self.title)

    def save(self, *args, **kwargs):
        self.slug = slugify(self.title)
        super(IconCollection, self).save(*args, **kwargs)

    def admin_thumbnail(self):
        icon = self.icons.all().first()
        if icon is not None:
            return mark_safe('<img src="{0}" height="{1}" alt="Icon">'
                             .format(icon.svg.url, DJEYM_YMAPS_ICONS_MAX_SIZE))
        else:
            return ""
    admin_thumbnail.short_description = _('Sample Icon')

    class Meta:
        ordering = ("title", "id")
        verbose_name = ungettext_lazy('Icon collection for markers',
                                      'Icon collections for markers', 5)
        verbose_name_plural = _('Icon collections for markers')

    def get_icon_count(self):
        return self.icons.all().count()
    get_icon_count.short_description = _('Icon count')

    def get_count_of_active_icons(self):
        return self.icons.filter(active=True).count()
    get_count_of_active_icons.short_description = _('Count of active icons')

    def get_export_file_btn(self):
        url = reverse('djeym:export_icon_collection',
                      kwargs={'slug': self.slug})
        return mark_safe('<a href="{0}" class="export_icon_collection_link">{1} '
                         '<div></div></a>'.format(url, _('Export Collection')))
    get_export_file_btn.short_description = _('Get a collection of icons')


class CustomMarkerIcon(models.Model):
    """Custom marker icon"""

    icon_collection = models.ForeignKey(
        IconCollection,
        verbose_name=_("Icon Collection"),
        related_name='icons',
        on_delete=models.CASCADE)

    title = models.CharField(
        _('Icon name'),
        max_length=60,
        default="",
        help_text=_('Example: Airport'))

    svg = models.FileField(
        _('Icon'),
        upload_to=make_upload_path,
        validators=[validate_svg],
        null=True,
        help_text=_('Only SVG files.'))

    size_width = models.PositiveSmallIntegerField(_('Icon width'), default=0)
    size_height = models.PositiveSmallIntegerField(_('Icon height'), default=0)

    offset_x = models.DecimalField(
        _('Offset by axis - X'),
        max_digits=3,
        decimal_places=1,
        default=Decimal(".0"),
        help_text=_('Left, right - First time is automatically calculated.'))

    offset_y = models.DecimalField(
        _('Offset by axis - Y'),
        max_digits=3,
        decimal_places=1,
        default=Decimal(".0"),
        help_text=_('Up, down - First time is automatically calculated.'))

    active = models.BooleanField(
        _('Active icon'),
        default=True,
        help_text=_("If the project uses 2-3 icons, "
                    "it makes sense to disable the rest to optimize the download."))

    slug = models.SlugField(max_length=255, blank=True, null=True)

    @property
    def upload_dir(self):
        return 'djeym_custom_icons'

    def __str__(self):
        return '{}'.format(self.title)

    def admin_thumbnail(self):
        if bool(self.svg):
            return mark_safe('<img src="{0}" height="{1}" alt="Icon">'
                             .format(self.svg.url, self.size_height))
        else:
            return ""
    admin_thumbnail.short_description = _('Icon')

    class Meta:
        ordering = ("title", "id")
        verbose_name = ungettext_lazy(
            'Icon for marker', 'Icons for markers', 5)
        verbose_name_plural = ungettext_lazy(
            'Icon for marker', 'Icons for markers', 2)

    def __init__(self, *args, **kwargs):
        super(CustomMarkerIcon, self).__init__(*args, **kwargs)
        self.__old_image = self.svg

    def save(self, *args, **kwargs):
        self.slug = slugify(self.title + "-" + self.icon_collection.title)
        old_image = self.__old_image
        new_image = self.svg
        if not bool(old_image) or (bool(old_image) and bool(new_image) and
                                   old_image.name != new_image.name):
            self.__old_image = new_image
            self.size_width = 0
            self.size_height = 0
            self.offset_x = Decimal(".0")
            self.offset_y = Decimal(".0")
        super(CustomMarkerIcon, self).save(*args, **kwargs)

    def clean(self):
        slug = slugify(self.title + "-" + self.icon_collection.title)
        icon_count = CustomMarkerIcon.objects.filter(slug=slug).count()
        if not bool(self.slug) and icon_count > 0:
            msg = _(
                'For the selected collection, there is already an icon with a similar name.')
            raise ValidationError({'title': msg})

    def get_collection_name(self):
        return "{}".format(self.icon_collection.title)
    get_collection_name.short_description = _('Collection name')

    def get_size(self):
        return '[{0},{1}]'.format(self.size_width, self.size_height)

    def get_offset(self):
        return '[{0:f},{1:f}]'.format(self.offset_x, self.offset_y)


class CounterID(models.Model):
    """Count of geo objects not a point type."""
    num_id = models.PositiveIntegerField('ID', default=1)


class Statistics(models.Model):
    """Statistics"""
    obj_type = models.CharField('Object type', max_length=255, default="")
    obj_id = models.PositiveIntegerField('Object ID', default=0)
    ip = models.GenericIPAddressField('IP-address', null=True)
    likes = models.BooleanField('Likes', default=False)
    timestamp = models.DateTimeField('Date and Time', default=timezone.now)

    def __str__(self):
        return '{}'.format(self.obj_type)

    class Meta:
        verbose_name = _('Record')
        verbose_name_plural = _('Statistics')


class LoadIndicator(models.Model):
    """Load Indicator"""
    svg = models.FileField(
        _('Icon'),
        upload_to=make_upload_path,
        validators=[validate_svg],
        null=True,
        help_text=_(
            'https://github.com/genkosta/django-editor-ymaps/blob/master/Spinner.zip')
    )

    title = models.CharField(_('Title'), unique=True,
                             max_length=60, default="")

    slug = models.SlugField(max_length=255, blank=True, null=True)

    @property
    def upload_dir(self):
        return 'djeym_load_indicators'

    def __str__(self):
        return '{}'.format(self.title)

    class Meta:
        ordering = ("title", "id")
        verbose_name = _('Load indicator')
        verbose_name_plural = _('Load indicators')

    def save(self, *args, **kwargs):
        self.slug = slugify(self.title)
        super(LoadIndicator, self).save(*args, **kwargs)

    def clean(self):
        slug = slugify(self.title)
        if slug == 'default':
            msg = _('Default - Reserved name for the indicator. Choose another name.')
            raise ValidationError({'title': msg})

    def admin_thumbnail(self):
        if bool(self.svg):
            return mark_safe('<img src="{0}" height="60" alt="Icon">'.format(self.svg.url))
        else:
            return ""
    admin_thumbnail.short_description = _('Icon')


# Signals ------------------------------------------------------------------------------------------

def custom_icon_cluster_size_correction(instance, **kwargs):
    """
    Custom Cluster Icons - Size correction and offset correction.
    Пользовательская иконки кластера - Корректировка размера и коррекция смещения.
    """
    image = instance.svg
    size_width = instance.size_width
    size_height = instance.size_height

    if bool(image) and (size_width == 0 or size_height == 0):
        sizes_svg = get_size_from_svg(image)
        width = sizes_svg["width"]
        height = sizes_svg["height"]
        sizes = get_size_correction(width, height)
        width = sizes[0]
        height = sizes[1]

        instance.size_width = width
        instance.size_height = height

        instance.offset_x = ((Decimal(width) / Decimal(2))
                             .quantize(Decimal(".0"))) * Decimal(-1)
        instance.offset_y = ((Decimal(height) / Decimal(2))
                             .quantize(Decimal(".0"))) * Decimal(-1)
        instance.save()


def custom_icon_marker_size_correction(instance, **kwargs):
    """
    Custom Marker Icon - Size correction and offset correction.
    Пользовательская иконка маркера - Корректировка размера и коррекция смещения.
    """
    image = instance.svg
    size_width = instance.size_width
    size_height = instance.size_height

    if bool(image) and (size_width == 0 or size_height == 0):
        sizes_svg = get_size_from_svg(image)
        width = sizes_svg["width"]
        height = sizes_svg["height"]
        sizes = get_size_correction(width, height)
        width = sizes[0]
        height = sizes[1]

        instance.size_width = width
        instance.size_height = height

        instance.offset_x = ((Decimal(width) / Decimal(2))
                             .quantize(Decimal(".0"))) * Decimal(-1)
        instance.offset_y = Decimal(height) * Decimal(-1)

        instance.save()


def placemark_update_json_code(instance, **kwargs):
    """Update json code in Placemark"""
    primary_json_code = json.loads(instance.json_code)
    json_code = json.loads(instance.json_code)

    if primary_json_code["id"] == 0:
        primary_json_code["id"] = instance.id
        json_code["id"] = instance.id

    primary_json_code = json.dumps(primary_json_code, ensure_ascii=False)
    json_code['geometry']['coordinates'] = json.loads(instance.coordinates)
    json_code["properties"]["id"] = instance.id
    json_code["properties"]["categoryID"] = instance.category.id
    json_code["properties"]["subCategoryIDs"] = [
        item.id for item in instance.subcategories.all()]
    json_code["properties"]["iconName"] = instance.icon_name
    icon_marker = CustomMarkerIcon.objects.get(slug=instance.icon_name)
    json_code["options"]['iconImageHref'] = icon_marker.svg.url
    json_code["options"]['iconImageSize'] = json.loads(icon_marker.get_size())
    json_code["options"]['iconImageOffset'] = json.loads(
        icon_marker.get_offset())
    json_code = json.dumps(json_code, ensure_ascii=False)

    if primary_json_code != json_code:
        instance.json_code = json_code
        instance.save()


def polyline_update_json_code(instance, **kwargs):
    """Update json code in Polyline"""
    primary_json_code = json.loads(instance.json_code)
    json_code = json.loads(instance.json_code)

    if primary_json_code["id"] == 0:
        counter_id = CounterID.objects.first()

        if counter_id is None:
            counter_id = CounterID.objects.create()

        num_id = counter_id.num_id
        primary_json_code["id"] = num_id
        json_code["id"] = num_id
        counter_id.num_id += 1
        counter_id.save()

    primary_json_code = json.dumps(primary_json_code, ensure_ascii=False)
    json_code['geometry']['coordinates'] = json.loads(instance.coordinates)
    json_code["properties"]["id"] = instance.pk
    json_code["properties"]["categoryID"] = instance.category.id
    json_code["options"]["strokeWidth"] = float(instance.stroke_width)
    json_code["options"]["strokeColor"] = instance.stroke_color
    json_code["options"]["strokeOpacity"] = float(instance.stroke_opacity)
    json_code = json.dumps(json_code, ensure_ascii=False)

    if primary_json_code != json_code:
        instance.json_code = json_code
        instance.save()


def polygon_update_json_code(instance, **kwargs):
    """Update json code in Polygon"""
    primary_json_code = json.loads(instance.json_code)
    json_code = json.loads(instance.json_code)

    if primary_json_code["id"] == 0:
        counter_id = CounterID.objects.first()

        if counter_id is None:
            counter_id = CounterID.objects.create()

        num_id = counter_id.num_id
        primary_json_code["id"] = num_id
        json_code["id"] = num_id
        counter_id.num_id += 1
        counter_id.save()

    primary_json_code = json.dumps(primary_json_code, ensure_ascii=False)
    json_code['geometry']['coordinates'] = json.loads(instance.coordinates)
    json_code["properties"]["id"] = instance.pk
    json_code["properties"]["categoryID"] = instance.category.pk
    json_code["options"]["strokeWidth"] = float(instance.stroke_width)
    json_code["options"]["strokeColor"] = instance.stroke_color
    json_code["options"]["strokeOpacity"] = float(instance.stroke_opacity)
    json_code["options"]["fillColor"] = instance.fill_color
    json_code["options"]["fillOpacity"] = float(instance.fill_opacity)
    json_code = json.dumps(json_code, ensure_ascii=False)

    if primary_json_code != json_code:
        instance.json_code = json_code
        instance.save()


def heatpoint_update_json_code(instance, **kwargs):
    """Update json code in HeatPoint"""
    primary_json_code = json.loads(instance.json_code)
    json_code = json.loads(instance.json_code)

    if primary_json_code["id"] == 0:
        primary_json_code["id"] = instance.id
        json_code["id"] = instance.id

    primary_json_code = json.dumps(primary_json_code, ensure_ascii=False)
    json_code['geometry']['coordinates'] = json.loads(instance.coordinates)
    json_code["properties"]["weight"] = int(instance.weight)
    json_code = json.dumps(json_code, ensure_ascii=False)

    if primary_json_code != json_code:
        instance.json_code = json_code
        instance.save()


def placemark_delete_statistics(instance, **kwargs):
    """Delete orphaned statistics - Placemark"""
    Statistics.objects.filter(obj_type='Point', obj_id=instance.pk).delete()


def polyline_delete_statistics(instance, **kwargs):
    """Delete orphaned statistics - Polyline"""
    Statistics.objects.filter(obj_type='LineString',
                              obj_id=instance.pk).delete()


def polygon_delete_statistics(instance, **kwargs):
    """Delete orphaned statistics - Polygon"""
    Statistics.objects.filter(obj_type='Polygon', obj_id=instance.pk).delete()


# Resize icons
post_save.connect(custom_icon_cluster_size_correction,
                  sender=CustomClusterIcon)
post_save.connect(custom_icon_marker_size_correction, sender=CustomMarkerIcon)

# Update json code
post_save.connect(placemark_update_json_code, sender=Placemark)
m2m_changed.connect(placemark_update_json_code,
                    sender=Placemark.subcategories.through)
post_save.connect(polyline_update_json_code, sender=Polyline)
post_save.connect(polygon_update_json_code, sender=Polygon)
post_save.connect(heatpoint_update_json_code, sender=HeatPoint)

# Clean old icons
pre_save.connect(cleaning_files_pre_save, sender=CustomClusterIcon)
pre_delete.connect(cleaning_files_pre_delete, sender=CustomClusterIcon)
pre_save.connect(cleaning_files_pre_save, sender=CustomMarkerIcon)
pre_delete.connect(cleaning_files_pre_delete, sender=CustomMarkerIcon)
pre_save.connect(cleaning_files_pre_save, sender=LoadIndicator)
pre_delete.connect(cleaning_files_pre_delete, sender=LoadIndicator)

# Clean old screenshots
pre_save.connect(cleaning_files_pre_save, sender=TileSource)
pre_delete.connect(cleaning_files_pre_delete, sender=TileSource)

# Delete orphaned presets
pre_delete.connect(placemark_delete_statistics, sender=Placemark)
pre_delete.connect(polyline_delete_statistics, sender=Polyline)
pre_delete.connect(polygon_delete_statistics, sender=Polygon)



# Generated by Django 2.1.7 on 2019-03-30 14:58

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('djeym', '0003_map_disable_indicator_animation'),
    ]

    operations = [
        migrations.AddField(
            model_name='map',
            name='animation_speed',
            field=models.CharField(choices=[('0.3', '0.3'), ('0.4', '0.4'), ('0.5', '0.5'), ('0.6', '0.6'), ('0.7', '0.7'), ('0.8', '0.8'), ('0.9', '0.9'), ('1', '1'), ('1.1', '1.1'), ('1.2', '1.2'), ('1.3', '1.3'), ('1.4', '1.4'), ('1.5', '1.5'), ('1.6', '1.6'), ('1.7', '1.7'), ('1.8', '1.8')], default='0.8', max_length=255, verbose_name='Load indicator animation speed'),
        ),
        migrations.AlterField(
            model_name='map',
            name='disable_indicator_animation',
            field=models.BooleanField(default=False, help_text='It may be useful for the abbreviation or logo of the company, if it does not make sense to animate them.', verbose_name='Disable loading indicator animation'),
        ),
    ]
